@@ -4,12 +4,15 @@
 *                                                                              *
 *                            F A L L B A C K S . H                             *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 2000                   *
+*                 Copyright (c) Joerg-R. Hill, October 2003                    *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: fallbacks.h,v 1.5 2000/12/10 15:06:57 jrh Exp $
+* $Id: fallbacks.h,v 1.6 2003/11/07 12:48:48 jrh Exp $
 * $Log: fallbacks.h,v $
+* Revision 1.6  2003/11/07 12:48:48  jrh
+* Release 2.4
+*
 * Revision 1.5  2000/12/10 15:06:57  jrh
 * Release 2.3
 *
@@ -27,10 +30,10 @@
 *
 */
 static String fallbacks[] = {
-  "Viewmol.webBrowser:                              netscape %s",
+  "Viewmol.webBrowser:                              mozilla %s",
   "Viewmol.Moloch:                                  moloch",
-  "Viewmol.Rayshade:                                rayshade",
-  "Viewmol.DisplayRLE:                              xv %s",
+  "Viewmol.Raytracer:                               x-povray +I%s +O%s +W%d +H%d",
+  "Viewmol.DisplayImage:                            xv %s",
 /* the following three resources are necessary to get
    the Indigo Magic Desktop look-and-feel */
   "Viewmol*sgiMode:                                 True",
@@ -44,9 +47,9 @@ static String fallbacks[] = {
   "Viewmol.model:                                   wire",
   "Viewmol.drawingMode:                             surface",
   "Viewmol.bondType:                                conjugated",
-  "Viewmol.sphereResolution:                        10",
+  "Viewmol.sphereResolution:                        20",
   "Viewmol.lineWidth:                               0",
-  "Viewmol.simplifyWhileRotating:                   True",
+  "Viewmol.simplifyWhileRotating:                   False",
   "Viewmol.interpolation:                           linear",
   "Viewmol.bondLength:                              %7.4f Ang",
   "Viewmol.bondAngle:                               %7.2f deg",
@@ -101,11 +104,12 @@ static String fallbacks[] = {
   "Viewmol*infoForm*text*columns:                   80",
   "Viewmol*annotation.highlightThickness:           0",
   "Viewmol.paperSize:                               A4",
-  "Viewmol.elementSortOrder:                        C,H,O,N,S",
+  "Viewmol.elementSortOrder:                        C,H,N,O,S",
   "Viewmol.viewer.font:                             variable",
   "Viewmol.spectrum.spectrum.font:                  variable",
   "Viewmol.history.history.font:                    variable",
   "Viewmol.MODiagram.MODiagram.font:                variable",
+  "Viewmol.MODiagram.MODiagram.greekFont:           -adobe-symbol-medium-r-normal--14-*",
   "Viewmol.viewer.background:                       white",
   "Viewmol.viewer.foreground:                       gray75",
   "Viewmol.spectrum.spectrum.background:            white",
@@ -116,7 +120,7 @@ static String fallbacks[] = {
   "Viewmol.MODiagram.MODiagram.foreground:          black",
   "Viewmol*foreground:                              black",
   "Viewmol.hasLZW:                                  False",
-  "Viewmol.language:                                english",
+  "Viewmol.language:                                en_US",
   "Viewmol.title:                                   Viewmol",
   "Viewmol.by:                                      by",
   "Viewmol.version:                                 Version",
@@ -225,8 +229,9 @@ static String fallbacks[] = {
   "Viewmol*_popup.annotate.accelerator:             Ctrl<Key>N",
   "Viewmol*_popup.annotate.acceleratorText:         Ctrl+N",
   "Viewmol*_popup.runScript.labelString:            Run script",
-  "Viewmol*_popup.runScript.accelerator:            Ctrl<Key>R",
-  "Viewmol*_popup.runScript.acceleratorText:        Ctrl+R",
+  "Viewmol*select.labelString:                      Select ...",
+  "Viewmol*select.accelerator:                      Ctrl<Key>R",
+  "Viewmol*select.acceleratorText:                  Ctrl+R",
   "Viewmol*_popup.hardcopy.labelString:             Save drawing ...",
   "Viewmol*_popup.hardcopy.mnemonic:                d",
   "Viewmol*_popup.hardcopy.accelerator:             Meta<Key>D",
@@ -403,6 +408,9 @@ static String fallbacks[] = {
   "Viewmol*spectrumForm*line_spectrum.labelString:  Line spectrum",
   "Viewmol*spectrumForm*gaussian_spectrum.labelString: Gaussian spectrum",
   "Viewmol*spectrumForm*setins.labelString:         Set weights for inelastic neutron scattering",
+  "Viewmol*spectrumForm*axisTop.labelString:        Wave numbers shown on top",
+  "Viewmol*spectrumForm*showGrid.labelString:       Show grid",
+  "Viewmol*spectrumForm*lineWidthLabel.labelString: Line width",
   "Viewmol*spectrumForm*temperature.labelString:    Temperature",
   "Viewmol*spectrumForm*amplitude.labelString:      Amplitude",
   "Viewmol*spectrumForm*scale.labelString:          Scale wave\\nnumbers",
@@ -428,8 +436,9 @@ static String fallbacks[] = {
   "Viewmol*printForm_popup.title:                   Save Drawing",
   "Viewmol*printForm*hpgl.labelString:              HPGL",
   "Viewmol*printForm*postscript.labelString:        PostScript",
-  "Viewmol*printForm*raytracer.labelString:         Rayshade",
+  "Viewmol*printForm*raytracer.labelString:         Povray",
   "Viewmol*printForm*tiff.labelString:              TIFF",
+  "Viewmol*printForm*png.labelString:               PNG",
   "Viewmol*printForm*landscape.labelString:         Landscape",
   "Viewmol*printForm*portrait.labelString:          Portrait",
   "Viewmol*printForm*papersize.labelString:         Paper size",
@@ -443,6 +452,7 @@ static String fallbacks[] = {
   "Viewmol*printForm*mac.labelString:               Macintosh",
   "Viewmol*printForm*none.labelString:              None",
   "Viewmol*printForm*compressionlabel.labelString:  TIFF compression",
+  "Viewmol*printForm*transparent.labelString:       Background transparent",
   "Viewmol*printForm*widthlabel.labelString:        Paper width in mm",
   "Viewmol*printForm*heightlabel.labelString:       Paper height in mm",
   "Viewmol*printForm*file.labelString:              File",
@@ -473,6 +483,7 @@ static String fallbacks[] = {
   "Viewmol*saveMoleculeForm_popup*title:            Save molecule",
   "Viewmol*saveMoleculeForm*car.labelString:        MSI car-file",
   "Viewmol*saveMoleculeForm*arc.labelString:        MSI arc-file",
+  "Viewmol*saveMoleculeForm*gauss.labelString:      Gaussian 98 input file",
   "Viewmol*saveMoleculeForm*mol.labelString:        MDL mol-file",
   "Viewmol*saveMoleculeForm*tm.labelString:         Turbomole file",
   "Viewmol*unitcellForm_popup.title:                Unit cell",
@@ -485,19 +496,22 @@ static String fallbacks[] = {
   "Viewmol*unitcellForm*kvalue.titleString:         k",
   "Viewmol*unitcellForm*lvalue.titleString:         l",
   "Viewmol*configurationForm_popup*title:           Configuration",
-  "Viewmol*configurationForm*english.labelString:   English",
-  "Viewmol*configurationForm*german.labelString:    German",
-  "Viewmol*configurationForm*russian.labelString:   Russian",
-  "Viewmol*configurationForm*french.labelString:    French",
-  "Viewmol*configurationForm*spanish.labelString:   Spanish",
+  "Viewmol*configurationForm*en_US.labelString:     English",
+  "Viewmol*configurationForm*fr.labelString:        French",
+  "Viewmol*configurationForm*de.labelString:        German",
+  "Viewmol*configurationForm*hu.labelString:        Hungarian",
+  "Viewmol*configurationForm*pl.labelString:        Polish",
+  "Viewmol*configurationForm*ru.labelString:        Russian",
+  "Viewmol*configurationForm*es.labelString:        Spanish",
   "Viewmol*configurationForm*browserLabel.labelString: Location of Web browser",
   "Viewmol*configurationForm*molochLabel.labelString:    Location of Moloch",
-  "Viewmol*configurationForm*rayshadeLabel.labelString:  Location of Rayshade",
-  "Viewmol*configurationForm*displayRLELabel.labelString:  Location of display program for RLE files",
+  "Viewmol*configurationForm*raytracerLabel.labelString:  Location of Povray",
+  "Viewmol*configurationForm*displayImageLabel.labelString:  Location of display program for images",
   "Viewmol.unknownParameter:                        Unknown command line parameter: %s",
   "Viewmol.selectFormat:                            Please select format.",
   "Viewmol.selectCompression:                       Please select compression method.",
   "Viewmol.TIFFSaved:                               Drawing saved to TIFF file %s.",
+  "Viewmol.PNGSaved:                                Drawing saved to PNG file %s.",
   "Viewmol.HPGLSaved:                               Drawing saved to HPGL file %s.",
   "Viewmol.PostscriptSaved:                         Drawing saved to Postscript file %s.",
   "Viewmol.RaytracerSaved:                          Drawing saved to Rayshade file %s.",
@@ -521,7 +535,7 @@ static String fallbacks[] = {
   "Viewmol.cannotDisplay:                           Unable to display manual.\nWeb browser does not start.",
   "Viewmol.noVisual:                                Unable to find a window suitable for OpenGL drawing.\\nThere might be an installation problem with OpenGL\\nor the X server has not been started with the\\ncorrect extensions.",
   "Viewmol.noRayshade:                              No location for Rayshade specified in resources.",
-  "Viewmol.noDisplay:                               No display program for RLE files specified in resources.",
+  "Viewmol.noDisplay:                               No display program for images specified in resources.",
   "Viewmol.unableToWriteFeedback:                   Cannot allocate enough memory to write drawing to file.",
   "Viewmol.wavenumberTitle:                         %s mode, %.6g cm-1",
   "Viewmol.selectAtomTitle:                         Please select atom by clicking on it.",
@@ -550,7 +564,7 @@ static String fallbacks[] = {
   "Viewmol.noEnergy:                                Cannot find energy in file %s.",
   "Viewmol.noMOselected:                            There is no MO selected for %s.\\nPlease select a MO in the energy level\\ndiagram, then try again.",
   "Viewmol.notChangeable:                           This coordinate cannot be changed\\nsince it is part of a ring.",
-  "Viewmol.notDefined:                              This torsion angle is not defined.",
+  "Viewmol.notDefined:                              This internal coordinate is not defined.",
   "Viewmol.formatNotRecognized:                     Cannot recognize format of file %s.",
   "Viewmol.wrongBrowser:                            Web browser cannot be found at specified location.",
   "Viewmol.wrongMoloch:                             Moloch cannot be found at specified location.",
@@ -561,6 +575,9 @@ static String fallbacks[] = {
   "Viewmol.deleteAll:                               Do you really want to delete\\nall molecules ?",
   "Viewmol.deleteOne:                               Do you really want to delete\\n%s ?",
   "Viewmol.unknownElement:                          Element %s is not known.",
+  "Viewmol.unknownElements:                         Elements %s are not known.",
+  "Viewmol.missingBasisset:                         The basis set for the atom(s) %s has not been specified.",
   "Viewmol.elementMenuPrefix:                       ",
+  "Viewmol.wrongPNGversion:                         Version %s of the PNG library is not supported. At least version 1.4 is needed.",
   NULL
 };
