@@ -4,12 +4,15 @@
 *                                                                              *
 *                                G E T R C . C                                 *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 2000                   *
+*                 Copyright (c) Joerg-R. Hill, October 2003                    *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: getrc.c,v 1.5 2000/12/10 15:07:37 jrh Exp $
+* $Id: getrc.c,v 1.6 2003/11/07 11:02:15 jrh Exp $
 * $Log: getrc.c,v $
+* Revision 1.6  2003/11/07 11:02:15  jrh
+* Release 2.4
+*
 * Revision 1.5  2000/12/10 15:07:37  jrh
 * Release 2.3
 *
@@ -57,6 +60,7 @@ extern int checkFile(char **);
 extern XVisualInfo *getVisualInfo(Display *, Visual *);
 extern void *getmem(size_t, size_t);
 extern void *expmem(void *, size_t, size_t);
+extern void fremem(void **);
 extern void osname(char *);
 extern int messgb(Widget, int, char *, struct PushButtonRow *, int);
 extern void loadAtomColors(XVisualInfo *, Colormap);
@@ -68,7 +72,7 @@ extern struct WINDOW windows[];
 extern struct OPTION *options;
 extern struct OPTION *output;
 extern Pixel stdcol[9];
-extern char webbrowser[MAXLENLINE], moloch[MAXLENLINE], rayshade[MAXLENLINE], displayRLE[MAXLENLINE];
+extern char webbrowser[MAXLENLINE], moloch[MAXLENLINE], raytracer[MAXLENLINE], displayImage[MAXLENLINE];
 extern int ne, nopt, noutput;
 extern double bndfac, temp, sphereres, denres, lineWidth;
 extern double paperHeight, paperWidth;
@@ -77,7 +81,7 @@ extern int lines, thermoUnits;
 extern int primitive, bondType, unit, automaticRecalculate;
 extern double level, hbondThreshold;
 extern char *formBondLength, *formBondAngle, *formTorsionAngle;
-extern int rgbMode, simplify, interp;
+extern int rgbMode, simplify, interp, showWarning;
 extern Widget topShell;
 extern XtAppContext app;
 
@@ -87,15 +91,22 @@ static char viewmolpath[MAXLENLINE];
 
 int getrc(void)
 {
+  struct OPTION *oldOptions;
   FILE *file;
   static struct PushButtonRow buttons1[] = {{"exit", GetMessageBoxButton, (XtPointer)0, NULL}};
-  char line[MAXLENLINE], rcfile[MAXLENLINE], *word, *oldLocale;
+  const char *locale;
+  char line[MAXLENLINE], rcfile[MAXLENLINE], *word;
+#ifdef DARWIN
+  char os[_SYS_NAMELEN+12];
+#else
   char os[SYS_NMLN+12];
-  size_t mne=110;
-  int nline=0, error;
+#endif
+  size_t mne=115;
+  int nline=0, error, i;
 
   debug=0;
   label=FALSE;
+  locale=setlocale(LC_ALL, "C");
 
 /* Find default model */
 
@@ -189,10 +200,10 @@ int getrc(void)
   windows[SPECTRUM].background=stdcol[WHITE];
   windows[SPECTRUM].mode=SPECTRUM_IR;
   word=getStringResource(topShell, "wavenumbers");
-  windows[SPECTRUM].left=atof(strtok(word, ":"))-10.;
-  windows[SPECTRUM].right=atof(strtok(NULL, ":"))+10.;
-  windows[SPECTRUM].bottom=110.0;
-  windows[SPECTRUM].top=-10.0;
+  windows[SPECTRUM].left=atof(strtok(word, ":"));
+  windows[SPECTRUM].right=atof(strtok(NULL, ":"));
+  windows[SPECTRUM].bottom=100.0;
+  windows[SPECTRUM].top=0.0;
   windows[SPECTRUM].near=0.0;
   windows[SPECTRUM].far=1.0;
   windows[HISTORY].foreground=stdcol[BLUE];
@@ -214,17 +225,17 @@ int getrc(void)
   denres=atof(getStringResource(topShell, "densityResolution"));
   getFormats();
 
-/* Get commands to start Web browser, Moloch, Rayshade, and display program for RLE files
+/* Get commands to start Web browser, Moloch, Rayshade, and display program for images
    and check if programs are accessible */
 
   word=getStringResource(topShell, "webBrowser");
   checkProgram(webbrowser, word);
   word=getStringResource(topShell, "Moloch");
   checkProgram(moloch, word);
-  word=getStringResource(topShell, "Rayshade");
-  checkProgram(rayshade, word);
-  word=getStringResource(topShell, "DisplayRLE");
-  checkProgram(displayRLE, word);
+  word=getStringResource(topShell, "Raytracer");
+  checkProgram(raytracer, word);
+  word=getStringResource(topShell, "DisplayImage");
+  checkProgram(displayImage, word);
 
 /* Get paper size */
 
@@ -289,7 +300,6 @@ int getrc(void)
 
   ne=0;
   osname(os);
-  oldLocale=setlocale(LC_ALL, "C");
   while (fgets(line, MAXLENLINE, file) != NULL)
   {
     nline++;
@@ -307,20 +317,32 @@ int getrc(void)
         if (options == NULL)
           options=(struct OPTION *)getmem(nopt+1, sizeof(struct OPTION));
         else
+        {
+          oldOptions=options;
           options=(struct OPTION *)expmem((void *)options, nopt+1, sizeof(struct OPTION));
+	    if (oldOptions != options)
+          {
+            /* Move pointer to identifier if expanding memory caused a move of the
+               entire options structure */
+            for (i=0; i<nopt+1; i++)
+		  options[i].identifier+=(char *)options-(char *)oldOptions;
+          }
+	  }
         strtok(line, " \t");
         strncpy(options[nopt].flag, strtok(NULL, " \t"), 8);
         storeCommand(options[nopt].command, strtok(NULL, " \t"), os);
         while ((word=strtok(NULL, " \t")) != NULL)
         {
-          strncat(options[nopt].command, " ", MAXLENLINE);
-          strncat(options[nopt].command, word, MAXLENLINE);
+          strncat(options[nopt].command, " ", MAXLENLINE-strlen(options[nopt].command)-1);
+          strncat(options[nopt].command, word, MAXLENLINE-strlen(options[nopt].command)-1);
         }
         if ((word=strtok(options[nopt].command, "\"")) != NULL)
         {
           options[nopt].identifier=strtok(NULL, "\"");
           (void)strtok(NULL, "\"");
         }
+	else
+	  options[nopt].identifier=NULL;
         nopt++;
       }
       else if (strstr(line, "output"))
@@ -334,8 +356,8 @@ int getrc(void)
         storeCommand(output[noutput].command, strtok(NULL, " \t"), os);
         while ((word=strtok(NULL, " \t")) != NULL)
         {
-          strncat(output[noutput].command, " ", MAXLENLINE);
-          strncat(output[noutput].command, word, MAXLENLINE);
+          strncat(output[noutput].command, " ", MAXLENLINE-strlen(output[noutput].command)-1);
+          strncat(output[noutput].command, word, MAXLENLINE-strlen(output[noutput].command)-1);
         }
         noutput++;
       }
@@ -410,39 +432,68 @@ int getrc(void)
     exit(-1);
   }
   fclose(file);
-  (void)setlocale(LC_ALL, oldLocale);
+  (void)setlocale(LC_ALL, locale);
   return(TRUE);
 }
 
 void setAtom(struct MOLECULE *mol)
 {
-  static struct PushButtonRow buttons[] = {{"exit", GetMessageBoxButton, (XtPointer)0, NULL}};
+  static struct PushButtonRow buttons[] = {{"exit", GetMessageBoxButton, (XtPointer)0, NULL},
+                                           {"continue", GetMessageBoxButton, (XtPointer)1, NULL}};
   XVisualInfo *visualInfo;
   Visual *vi;
   Colormap colormap;
   char *word, line[MAXLENLINE];
-  register int i, j;
+  char *missing=NULL, test[5];
+  register int i, j, k;
 
-  mol->existsUnitCell=FALSE;
+  k=1;
   for (i=0; i<mol->na; i++)
   {
-    if (!mol->existsUnitCell && !strncmp(mol->atoms[i].name, "Uc", 2)) mol->existsUnitCell=TRUE;
     for (j=0; j<ne; j++)
     {
       if (!strncmp(mol->atoms[i].name, elements[j].symbol, 2))
       {
         mol->atoms[i].rad=elements[j].rad;
-	  mol->atoms[i].radScale=1.0;
+        mol->atoms[i].radScale=1.0;
         mol->atoms[i].element=&elements[j];
-	  break;
+        break;
       }
     }
     if (mol->atoms[i].element == NULL)
     {
-      word=getStringResource(topShell, "unknownElement");
-      sprintf(line, word, mol->atoms[i].name);
-      messgb(topShell, 3, line, buttons, 1);
-      exit(-1);
+      if (missing == NULL)
+      {
+        k+=4;
+        missing=(char *)getmem((size_t)k, sizeof(char));
+        strncat(missing, mol->atoms[i].name, 2);
+        strncat(missing, ", ", 2);
+      }
+      else
+      {
+	strcpy(test, mol->atoms[i].name);
+	strcat(test, ", ");
+        if (!strstr(missing, test))
+	{
+          k+=4;
+          missing=(char *)expmem((void *)missing, (size_t)k, sizeof(char));
+          strncat(missing, mol->atoms[i].name, 2);
+          strncat(missing, ", ", 2);
+        }
+      }
+      mol->atoms[i].radScale=1.0;
+      mol->atoms[i].mass=0.0;
+      mol->atoms[i].nelectrons=0;
+      mol->atoms[i].neutronScatterfac=0.0;
+      for (j=0; j<ne; j++)
+      {
+        if (!strncmp(elements[j].symbol, "??", 2))
+        {
+          mol->atoms[i].element=&elements[j];
+          mol->atoms[i].rad=elements[j].rad;;
+          break;
+        }
+      }
     }
     for (j=0; j<LASTELEMENT; j++)
     {
@@ -451,10 +502,32 @@ void setAtom(struct MOLECULE *mol)
         mol->atoms[i].mass=isotope[j][0];
         mol->atoms[i].nelectrons=electrons[j];
         mol->atoms[i].neutronScatterfac=0.0;
-	  break;
+        break;
       }
     }
   }
+  if (k > 1)
+  {
+    if (showWarning)
+    {
+      if (k > 5)
+        word=getStringResource(topShell, "unknownElements");
+      else
+        word=getStringResource(topShell, "unknownElement");
+      missing[strlen(missing)-2]='\0';
+      sprintf(line, word, missing);
+      fremem((void **)&missing);
+      if (!messgb(topShell, 3, line, buttons, 2))
+        exit(-1);
+    }
+  }
+  if (mol->unitcell)
+  {
+    for (i=0; i<ne; i++)
+      if (!strncmp(elements[i].symbol, "Uc", 2)) break;
+    for (j=0; j<mol->unitcell->nc; j++)
+      mol->unitcell->corners[j].element=&elements[i];
+  }
   setBond(mol);
   if (!rgbMode)
   {
@@ -499,10 +572,16 @@ int getColor(GLfloat *p)
 
   if ((word=strtok(NULL, " \t")) == NULL) return(TRUE);
   p[0]=(float)atof(word);
+  if (p[0] < 0.0) p[0]=0.0;
+  if (p[0] > 1.0) p[0]=1.0;
   if ((word=strtok(NULL, " \t")) == NULL) return(TRUE);
   p[1]=(float)atof(word);
+  if (p[1] < 0.0) p[1]=0.0;
+  if (p[1] > 1.0) p[1]=1.0;
   if ((word=strtok(NULL, " \t")) == NULL) return(TRUE);
   p[2]=(float)atof(word);
+  if (p[2] < 0.0) p[2]=0.0;
+  if (p[2] > 1.0) p[2]=1.0;
   p[3]=0.0;
   return(FALSE);
 }
@@ -544,6 +623,7 @@ void storeCommand(char *target, char *value, char *replacement)
         *t++=*p++;
     }
   }
+  *t='\0';
 }
 
 void checkProgram(char *program, char *word)
