@@ -4,12 +4,15 @@
 *                                                                              *
 *                                 I N I T . C                                  *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 2000                   *
+*                 Copyright (c) Joerg-R. Hill, October 2003                    *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: init.c,v 1.5 2000/12/10 15:09:19 jrh Exp $
+* $Id: init.c,v 1.6 2003/11/07 11:04:56 jrh Exp $
 * $Log: init.c,v $
+* Revision 1.6  2003/11/07 11:04:56  jrh
+* Release 2.4
+*
 * Revision 1.5  2000/12/10 15:09:19  jrh
 * Release 2.3
 *
@@ -26,6 +29,9 @@
 * Initial revision
 *
 */
+#include<sys/types.h>
+#include<dirent.h>
+#include<fnmatch.h>
 #include<stdio.h>
 #include<stdlib.h>
 #include<X11/Xlib.h>
@@ -60,6 +66,12 @@ void setCursor(Widget, unsigned int);
 void setMenuItem(int, int);
 void blendToPicture(void);
 struct MenuItem *makeMoleculesMenu(struct MenuItem *, int);
+struct MenuItem *makeScriptsMenu(struct MenuItem *);
+#if defined LINUX || defined DARWIN
+int matchScripts(const struct dirent *);
+#else
+int matchScripts(struct dirent *);
+#endif
 void setMenu(struct MOLECULE *);
 void initPython(int, char **);
 
@@ -71,10 +83,11 @@ extern void setAtom(struct MOLECULE *);
 extern void inert(struct MOLECULE *);
 extern int  makeConnectivity(struct MOLECULE *, int, int);
 extern void *getmem(size_t, size_t);
+extern void *expmem(void *, size_t, size_t);
 extern void fremem(void **);
 extern void storeColor(Pixel, struct WINDOW *, int);
 extern void redraw(int);
-extern void drawBackground(int, Pixel, float);
+extern void drawBackground(int, Pixel, double);
 extern int StringWidth(XFontStruct *, char *);
 extern void deleteFontList(GLuint, XFontStruct *);
 extern XFontStruct *makeRasterFont(Display *, char *, GLuint *);
@@ -85,7 +98,7 @@ extern void reshape(Widget, caddr_t, XmDrawingAreaCallbackStruct *);
 extern void hardcopyCallback(Widget, caddr_t, XmAnyCallbackStruct *);
 extern Widget makeMenu(Widget, int, char *, struct MenuItem *);
 extern void processInput(Widget, caddr_t, XmDrawingAreaCallbackStruct *);
-extern void (*drawString)(char *, double, double, double, double, GLuint);
+extern void (*drawString)(char *, double, double, double, double, GLuint, char);
 extern void setWindowColor(int, Pixel, const float *);
 extern void getImaginaryMode(Widget, caddr_t, caddr_t);
 extern void selectMolecule(Widget, caddr_t, XmToggleButtonCallbackStruct *);
@@ -105,6 +118,11 @@ extern void initHistoryModule(void);
 extern void initEnergyLevelModule(void);
 extern void initLightModule(void);
 extern void initLabelModule(void);
+extern void runScript(Widget, caddr_t, XmAnyCallbackStruct *);
+extern void startModule(Widget, caddr_t, XmAnyCallbackStruct *);
+extern void addBoundaryAtoms(struct MOLECULE *);
+extern void getScreenCoordinates(double, double, double, double *, double *,
+                                             double *);
 
 extern struct MOLECULE *molecules;
 extern struct WINDOW windows[];
@@ -115,8 +133,7 @@ extern float *transObject, *rotObject;
 extern int nmolecule;
 extern int label, showForces, noutput, nUndo;
 extern int showUnitCell, showInertia, swapBuffers;
-extern int annotate;
-extern char language[];
+extern char language[], *moduleName;
 
 extern XVisualInfo *vi;
 
@@ -133,7 +150,7 @@ int initViewer(int argc, char **argv, Widget widget)
   setCursor(widget, XC_watch);
   if ((string=getStringResource(widget, "font")) == NULL) string=def;
   windows[VIEWER].font=makeRasterFont(XtDisplay(widget), string,
-						  &windows[VIEWER].GLfontId);
+                                      &windows[VIEWER].GLfontId);
   createFileselect();
 
   transObject=(float *)getmem(MOLECULES, 3*sizeof(float));
@@ -147,22 +164,22 @@ int initViewer(int argc, char **argv, Widget widget)
   {
     for (j=0; j<TEXTURE_HEIGHT; j++)
     {
-	if (i % 4)
-	{
-	  texture[i][j][0]=(GLubyte)255;
-	  texture[i][j][1]=(GLubyte)255;
-	  texture[i][j][2]=(GLubyte)255;
-	}
-	else
-	{
-	  texture[i][j][0]=(GLubyte)0;
-	  texture[i][j][1]=(GLubyte)0;
-	  texture[i][j][2]=(GLubyte)0;
-	}
+      if (i % 4)
+      {
+        texture[i][j][0]=(GLubyte)255;
+        texture[i][j][1]=(GLubyte)255;
+        texture[i][j][2]=(GLubyte)255;
+      }
+      else
+      {
+        texture[i][j][0]=(GLubyte)0;
+        texture[i][j][1]=(GLubyte)0;
+        texture[i][j][2]=(GLubyte)0;
+      }
     }
   }
   glTexImage2D(GL_TEXTURE_2D, 0, 3, TEXTURE_WIDTH, TEXTURE_HEIGHT,
-		   0, GL_RGB, GL_UNSIGNED_BYTE, texture);
+               0, GL_RGB, GL_UNSIGNED_BYTE, texture);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
@@ -172,8 +189,11 @@ int initViewer(int argc, char **argv, Widget widget)
   if (!input(argc, argv))
   {
     setMenuItem(VIEWER_SELECT, False);
+/*  setMenuItem(VIEWER_BONDTYPE, False);
     setMenuItem(VIEWER_INERTIA, False);
     setMenuItem(VIEWER_LABEL, False);
+    setMenuItem(VIEWER_THERMODYNAMICS, False);*/
+    setMenuItem(VIEWER_HARDCOPY, False);
     setMenuItem(VIEWER1_SAVE, False);
     setMenuItem(VIEWER1_DELETE, False);
     setMenuItem(VIEWER1_BUILD, False);
@@ -186,9 +206,10 @@ int initViewer(int argc, char **argv, Widget widget)
     mol=&molecules[i];
     setAtom(mol);
     inert(mol);
+    if (mol->unitcell) addBoundaryAtoms(mol);
     if (!makeConnectivity(mol, TRUE, TRUE)) return(FALSE);
     box=windows[VIEWER].far > box ? windows[VIEWER].far : box;
-    if (!mol->existsUnitCell) showUnitCell=FALSE;
+    if (!mol->unitcell) showUnitCell=FALSE;
   }
   setWindowSize(box);
   setWindowTitle(windows[VIEWER].widget, molecules[windows[VIEWER].set].title);
@@ -200,17 +221,21 @@ int initViewer(int argc, char **argv, Widget widget)
 
 Widget makeViewerMenu(Widget widget)
 {
-  static struct MenuItem *submenu=NULL;
+  static struct MenuItem *moleculesMenu=NULL;
+  static struct MenuItem *scriptsMenu=NULL;
   Widget menu;
   struct MOLECULE *mol;
   register int i;
 
   if (nmolecule > 1)
   {
-    submenu=makeMoleculesMenu(submenu, VIEWER);
+    moleculesMenu=makeMoleculesMenu(moleculesMenu, VIEWER);
     i=VIEWER_SELECT;
-    viewerMenu[i & 0xff].subitems=submenu;
+    viewerMenu[i & 0xff].subitems=moleculesMenu;
   }
+  scriptsMenu=makeScriptsMenu(scriptsMenu);
+  i=VIEWER_RUNSCRIPT;
+  viewerMenu[i & 0xff].subitems=scriptsMenu;
   menu=makeMenu(widget, XmMENU_POPUP, "viewmol", viewerMenu);
   if (molecules == NULL)
     mol=NULL;
@@ -273,7 +298,7 @@ void makeSpectrumMenu(void)
   mol=&molecules[imol];
   if (mol->imag != 0)
   {
-    if (submenu1 != NULL) fremem((void *)&submenu1);
+    if (submenu1 != NULL) fremem((void **)&submenu1);
     submenu1=(struct MenuItem *)getmem(mol->imag+1, sizeof(struct MenuItem));
     label=(char *)getmem(mol->imag, 10);
     for (i=0; i<mol->imag; i++)
@@ -297,7 +322,7 @@ void makeSpectrumMenu(void)
     spectrumMenu[i & 0xff].subitems=submenu2;
   }
   windows[SPECTRUM].menu=makeMenu(windows[SPECTRUM].widget,
-					    XmMENU_POPUP, "spectrum_menu", spectrumMenu);
+                                  XmMENU_POPUP, "spectrum_menu", spectrumMenu);
   setMenuItem(VIEWER_SPECTRUM, False);
   if (nmolecule <= 1)
     setMenuItem(SPECTRUM_SELECT, False);
@@ -319,6 +344,7 @@ void makeSpectrumMenu(void)
 void initHistory(Widget w, caddr_t client_data, caddr_t call_data)
 {
   Pixel pixel;
+  double xs, ys, zs;
   char *string, line[MAXLENLINE];
 
   makeWindow(HISTORY, "history", (void *(*)())drawHistory);
@@ -339,6 +365,9 @@ void initHistory(Widget w, caddr_t client_data, caddr_t call_data)
   string=getStringResource(XtParent(windows[HISTORY].widget), "title");
   sprintf(line, string, molecules[windows[HISTORY].set].title);
   setWindowTitle(windows[HISTORY].widget, line);
+  redraw(HISTORY);
+  getScreenCoordinates((double)(molecules[windows[HISTORY].set].nhist), 0.0, 0.0, &xs, &ys, &zs);
+  windows[HISTORY].mouseX=(int)xs;
   redraw(VIEWER);
 }
 
@@ -354,7 +383,7 @@ void makeHistoryMenu(void)
     historyMenu[i & 0xff].subitems=submenu;
   }
   windows[HISTORY].menu=makeMenu(windows[HISTORY].widget,
-					   XmMENU_POPUP, "history_menu", historyMenu);
+                                 XmMENU_POPUP, "history_menu", historyMenu);
   setMenuItem(VIEWER_OPTIMIZATION, False);
   if (nmolecule <= 1)
   {
@@ -387,6 +416,9 @@ void initMODiagram(Widget w, caddr_t client_data, caddr_t call_data)
   string=getStringResource(windows[MO].widget, "font");
   windows[MO].font=makeRasterFont(XtDisplay(windows[MO].widget), string,
                                   &windows[MO].GLfontId);
+  string=getStringResource(windows[MO].widget, "greekFont");
+  windows[MO].greekFont=makeRasterFont(XtDisplay(windows[MO].widget), string,
+                                       &windows[MO].GLgreekFontId);
   if (windows[VIEWER].set >= 0)
     windows[MO].set=windows[VIEWER].set;
   else
@@ -406,6 +438,8 @@ void initMODiagram(Widget w, caddr_t client_data, caddr_t call_data)
     windows[MO].bottom=atof(strtok(string, ":"));
     windows[MO].top=atof(strtok(NULL, ":"));
   }
+  windows[MO].horizontalMargin=0;
+  windows[MO].verticalMargin=0;
   string=getStringResource(XtParent(windows[MO].widget), "title");
   sprintf(line, string, molecules[windows[MO].set].title);
   setWindowTitle(windows[MO].widget, line);
@@ -423,7 +457,7 @@ void makeMODiagramMenu(void)
     MODiagramMenu[i & 0xff].subitems=submenu;
   }
   windows[MO].menu=makeMenu(windows[MO].widget, XmMENU_POPUP,
-				    "MODiagram_menu", MODiagramMenu);
+                            "MODiagram_menu", MODiagramMenu);
   setMenuItem(MO_TRANSITION, False);
   setMenuItem(MO_ZOOMOUT, False);
   setMenuItem(VIEWER_ENERGYLEVEL, False);
@@ -458,12 +492,12 @@ void makeWindow(int which, char *name, void *drawFunc())
   XtVaGetValues(topShell, XtNvisual, &vi, XtNdepth, &depth, XtNcolormap, &colormap, NULL);
   shell=XtVaCreatePopupShell(name, applicationShellWidgetClass, topShell,
                              XtNvisual, vi,
-				     XtNdepth, depth,
-				     XtNcolormap, colormap,
+                             XtNdepth, depth,
+                             XtNcolormap, colormap,
                              NULL, 0);
   windows[which].widget=XtVaCreateManagedWidget(name, xmDrawingAreaWidgetClass, shell,
                                                 XtNvisual, vi,
-								XmNtranslations, XtParseTranslationTable(translations),
+                                                XmNtranslations, XtParseTranslationTable(translations),
                                                 XmNmarginHeight, 0,
                                                 XmNmarginWidth, 0,
                                                 NULL);
@@ -506,12 +540,12 @@ void showTitle(Widget widget,  caddr_t client_data, XmDrawingAreaCallbackStruct
   Display *display;
   XFontStruct *font;
   GLuint GLfont;
-  char viewmol[] = PROGRAM;
+  char viewmol[8] = PROGRAM;
   char *version, *by;
   char author[21] = "Jrg-Rdiger Hill";
   char versionLine[30];
-  const float yellow[4]  = {1.0, 1.0,  0.0, 0.0};
-  const float black[4]   = {0.0, 0.0,  0.0, 0.0};
+  const float yellow[4]  = {1.0, 1.0, 0.0, 0.0};
+  const float black[4]   = {0.0, 0.0, 0.0, 0.0};
   float save[3];
   int w, cyrillic=FALSE;
 
@@ -538,7 +572,7 @@ void showTitle(Widget widget,  caddr_t client_data, XmDrawingAreaCallbackStruct
   windows[VIEWER].background_rgb[1]=0.0;
   windows[VIEWER].background_rgb[2]=1.0;
   windows[VIEWER].smooth=2;
-  drawBackground(VIEWER, stdcol[SKYBLUE], 0.0);
+  drawBackground(VIEWER, stdcol[SKYBLUE], (double)0.0);
   windows[VIEWER].background_rgb[0]=save[0];
   windows[VIEWER].background_rgb[1]=save[1];
   windows[VIEWER].background_rgb[2]=save[2];
@@ -567,7 +601,7 @@ void showTitle(Widget widget,  caddr_t client_data, XmDrawingAreaCallbackStruct
       font=makeRasterFont(display, "fixed", &GLfont);
   }
   w=StringWidth(font, viewmol);
-  (*drawString)(viewmol, (GLdouble)(width-w)/2.e0, 2.e0/3.e0*height, 0.0, 0.0, GLfont);
+  (*drawString)(viewmol, (GLdouble)(width-w)/2.e0, 2.e0/3.e0*height, 0.0, 0.0, GLfont, 'l');
   deleteFontList(GLfont, font);
   setWindowColor(FOREGROUND, stdcol[BLACK], black);
   if (cyrillic)
@@ -588,12 +622,12 @@ void showTitle(Widget widget,  caddr_t client_data, XmDrawingAreaCallbackStruct
       font=makeRasterFont(display, "fixed", &GLfont);
   }
   w=StringWidth(font, versionLine);
-  (*drawString)(versionLine, (GLdouble)(width-w)/2.e0, height/2.e0, 0.0, 0.0, GLfont);
+  (*drawString)(versionLine, (GLdouble)(width-w)/2.e0, height/2.e0, 0.0, 0.0, GLfont, 'l');
   by=getStringResource(topShell, "by");
   w=StringWidth(font, by);
-  (*drawString)(by, (GLdouble)(width-w)/2.e0, height/2.5e0, 0.0, 0.0, GLfont);
+  (*drawString)(by, (GLdouble)(width-w)/2.e0, height/2.5e0, 0.0, 0.0, GLfont, 'l');
   w=StringWidth(font, author);
-  (*drawString)(author, (GLdouble)(width-w)/2.e0, height/3.e0, 0.0, 0.0, GLfont);
+  (*drawString)(author, (GLdouble)(width-w)/2.e0, height/3.e0, 0.0, 0.0, GLfont, 'l');
   deleteFontList(GLfont, font);
   glXSwapBuffers(XtDisplay(windows[VIEWER].widget), XtWindow(windows[VIEWER].widget));
 }
@@ -630,7 +664,7 @@ void blendToPicture(void)
   register int i, j, k;
 
   XtVaGetValues(windows[VIEWER].widget, XmNwidth, &width, XmNheight, &height,
-		    NULL);
+                NULL);
   incx=(GLsizei)width/ngrid;
   incy=(GLsizei)height/ngrid;
   if (width % ngrid != 0) incx++;
@@ -687,8 +721,8 @@ struct MenuItem *makeMoleculesMenu(struct MenuItem *submenu, int window)
   int offset;
   register int i;
 
-  if (submenu != NULL) fremem((void *)&submenu);
-  if (windows[window].selectMenu != NULL) fremem((void *)&windows[window].selectMenu);
+  if (submenu != NULL) fremem((void **)&submenu);
+  if (windows[window].selectMenu != NULL) fremem((void **)&windows[window].selectMenu);
   offset=nmolecule;
   if (window == VIEWER) offset+=2;
   submenu=(struct MenuItem *)getmem(offset+1, sizeof(struct MenuItem));
@@ -729,9 +763,106 @@ struct MenuItem *makeMoleculesMenu(struct MenuItem *submenu, int window)
   return(submenu);
 }
 
+struct MenuItem *makeScriptsMenu(struct MenuItem *submenu)
+{
+  DIR *d;
+  struct dirent **namelist=NULL;
+  static char sel[]="select", sep[]="sep";
+  char path[PATH_MAX], cmd[PATH_MAX], module[PATH_MAX], *word;
+  int n;
+  register int i, j;
+
+  if (submenu != NULL) return(submenu); /* fremem((void **)&submenu); */
+  strncpy(path, getenv("VIEWMOLPATH"), PATH_MAX-1);
+  strncat(path, "/scripts", PATH_MAX-strlen(path));
+  if ((d=opendir(path)) != NULL)
+  {
+    closedir(d);
+    strncpy(cmd, "import sys\nsys.path.insert(0, \"", PATH_MAX-1);
+    strncat(cmd, path, PATH_MAX-strlen(cmd)-1);
+    strncat(cmd, "\")", PATH_MAX-strlen(cmd)-1);
+    PyRun_SimpleString(cmd);
+    n=scandir(path, &namelist, matchScripts, alphasort);
+    if (n > 0)
+    {
+      submenu=(struct MenuItem *)getmem(n+3, sizeof(struct MenuItem));
+      j=0;
+      for (i=0; i<n; i++)
+      {
+        strncpy(module, namelist[i]->d_name, PATH_MAX-1);
+        word=strstr(module, ".py");
+        if (word) *word='\0';
+        sprintf(cmd, "try:\n  import %s\nexcept:\n  print \"Error loading module %s.\"\nelse:\n  module%2.2d=%s.%s()\n  module%2.2d.register(\"%s\")\n",
+                module, module, i, module, module, i, language);
+	moduleName='\0';
+        PyRun_SimpleString(cmd);
+        if (moduleName)
+        {
+          submenu[j].label=moduleName;
+          submenu[j].class=&xmPushButtonGadgetClass;
+          submenu[j].callback=startModule;
+          submenu[j].callback_data=(XtPointer)j;
+          submenu[j].widget=(Widget)NULL;
+          submenu[j++].subitems=NULL;
+        }
+      }
+      submenu[j].label=sep;
+      submenu[j].class=&xmSeparatorGadgetClass;
+      submenu[j].callback=(XtPointer)0;
+      submenu[j].callback_data=(XtPointer)0;
+      submenu[j].widget=(Widget)NULL;
+      submenu[j++].subitems=NULL;
+    }
+    else
+    {
+      submenu=(struct MenuItem *)getmem(1, sizeof(struct MenuItem));
+      j=0;
+    }
+  }
+  else
+  {
+    submenu=(struct MenuItem *)getmem(1, sizeof(struct MenuItem));
+    j=0;
+  }
+  submenu[j].label=sel;
+  submenu[j].class=&xmPushButtonGadgetClass;
+  submenu[j].callback=runScript;
+  submenu[j].callback_data=(XtPointer)0;
+  submenu[j].widget=(Widget)NULL;
+  submenu[j++].subitems=NULL;
+  submenu[j].label=NULL;
+  fremem((void **)&namelist);
+  return(submenu);
+}
+
+#if defined LINUX || defined DARWIN
+int matchScripts(const struct dirent *d)
+#else
+int matchScripts(struct dirent *d)
+#endif
+{
+  return(!fnmatch("*.py", d->d_name, 0));
+}
+
 void setMenu(struct MOLECULE *mol)
 {
-  if (!mol || mol->basisset == NULL)                        setMenuItem(VIEWER_WAVEFUNCTION, False);
+  if (!mol)
+  {
+    setMenuItem(VIEWER_BONDTYPE, False);
+    setMenuItem(VIEWER_THERMODYNAMICS, False);
+    setMenuItem(VIEWER_INERTIA, False);
+    setMenuItem(VIEWER_LABEL, False);
+    setMenuItem(VIEWER_ANNOTATE, False);
+  }
+  else 
+  {
+    setMenuItem(VIEWER_BONDTYPE, True);
+    setMenuItem(VIEWER_THERMODYNAMICS, True);
+    setMenuItem(VIEWER_INERTIA, True);
+    setMenuItem(VIEWER_LABEL, True);
+    setMenuItem(VIEWER_ANNOTATE, True);
+  }
+  if (!mol || (mol->basisset == NULL && mol->gridObjects == NULL)) setMenuItem(VIEWER_WAVEFUNCTION, False);
   else                                                      setMenuItem(VIEWER_WAVEFUNCTION, True);
   if (!mol || mol->orbitals == NULL || windows[MO].widget)  setMenuItem(VIEWER_ENERGYLEVEL, False);
   else                                                      setMenuItem(VIEWER_ENERGYLEVEL, True);
@@ -741,7 +872,7 @@ void setMenu(struct MOLECULE *mol)
   else                                                      setMenuItem(VIEWER_FORCES, True);
   if (!mol || mol->nmodes == 0 || windows[SPECTRUM].widget) setMenuItem(VIEWER_SPECTRUM, False);
   else                                                      setMenuItem(VIEWER_SPECTRUM, True);
-  if (!mol || !mol->existsUnitCell)
+  if (!mol || !mol->unitcell)
     setMenuItem(VIEWER_UNITCELL, False);
   else
   {
