@@ -4,12 +4,15 @@
 *                                                                              *
 *                               M K C O N N . C                                *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 2000                   *
+*                 Copyright (c) Joerg-R. Hill, October 2003                    *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: mkconn.c,v 1.5 2000/12/10 15:11:48 jrh Exp $
+* $Id: mkconn.c,v 1.6 2003/11/07 11:07:02 jrh Exp $
 * $Log: mkconn.c,v $
+* Revision 1.6  2003/11/07 11:07:02  jrh
+* Release 2.4
+*
 * Revision 1.5  2000/12/10 15:11:48  jrh
 * Release 2.3
 *
@@ -36,15 +39,16 @@
 
 extern struct MOLECULE *molecules;
 extern struct WINDOW windows[];
-extern double amplitude, forceScale, hbondThreshold;
+extern double amplitude, forceScale, hbondThreshold, lineWidth;
 extern int nmolecule, ne;
 extern int debug, bondType;
 extern Widget topShell;
 
+double centerMolecule(struct MOLECULE *);
 void setWindowSize(double);
 
 extern struct SAVE *saveGeometry(struct MOLECULE *);
-extern double bondLength(struct MOLECULE *, int, int);
+extern double bondLength(struct ATOM *, int, int);
 extern void GetMessageBoxButton(Widget, XtPointer, caddr_t);
 extern char *getStringResource(Widget, char *);
 extern void *getmem(size_t, size_t);
@@ -60,34 +64,13 @@ extern void redraw(int);
 int makeConnectivity(struct MOLECULE *mol, int setWindow, int saveCoord)
 {
   Dimension width, height;
-  double mass, r1, r2, box, xpix, ypix;
-  int *connectivity, maxBondsPerAtom=8, uc=mol->existsUnitCell, found, nbonds;
+  double r1, r2, box, xpix, ypix;
+  int *connectivity, maxBondsPerAtom=8, found, nbonds;
   size_t mnb=20;
   register int i, j, k, l;
 
-  mol->transx=0.0;
-  mol->transy=0.0;
-  mol->transz=0.0;
-  mass=0.0;
-  for (i=0; i<mol->na; i++)
-  {
-    mol->transx+=mol->atoms[i].mass*mol->atoms[i].x;
-    mol->transy+=mol->atoms[i].mass*mol->atoms[i].y;
-    mol->transz+=mol->atoms[i].mass*mol->atoms[i].z;
-    mass+=mol->atoms[i].mass;
-  }
-  mol->transx=-mol->transx/mass;
-  mol->transy=-mol->transy/mass;
-  mol->transz=-mol->transz/mass;
-  if (debug == 1)
-    printf("Shifting molecule by %f %f %f.\n", mol->transx, mol->transy,
-           mol->transz);
-  for (i=0; i<mol->na; i++)
-  {
-    mol->atoms[i].x+=mol->transx;
-    mol->atoms[i].y+=mol->transy;
-    mol->atoms[i].z+=mol->transz;
-  }
+  box=centerMolecule(mol);
+  if (setWindow) setWindowSize(box);
   if (saveCoord)
   {
     if (mol->coord != NULL) fremem((void **)&(mol->coord));
@@ -96,69 +79,62 @@ int makeConnectivity(struct MOLECULE *mol, int setWindow, int saveCoord)
 
   repeat:
   connectivity=(int *)getmem((size_t)(maxBondsPerAtom*mol->na), sizeof(int));
-  box=0.0;
   for (i=0; i<mol->na; i++)
   {
     mol->atoms[i].nbonds=0;
     mol->atoms[i].hbonds=0;
-    if (strncmp(mol->atoms[i].name, "Uc", 2))
+    for (j=0; j<mol->na; j++)
     {
-      for (j=0; j<mol->na; j++)
+      if (i != j)
       {
-        if (i != j && strncmp(mol->atoms[j].name, "Uc", 2))
+        r1=bondLength(mol->atoms, i, j);
+        r2=mol->atoms[i].radScale*mol->atoms[i].rad
+          +mol->atoms[j].radScale*mol->atoms[j].rad;
+        if (r1 < r2 && strncmp(mol->atoms[i].element->symbol, "??", 2) &&
+            strncmp(mol->atoms[j].element->symbol, "??", 2))
         {
-          r1=bondLength(mol, i, j);
-          r2=mol->atoms[i].radScale*mol->atoms[i].rad
-	      +mol->atoms[j].radScale*mol->atoms[j].rad;
-          box=box > r1+r2 ? box : r1+r2;
-          if (r1 < r2)
+          mol->atoms[i].nbonds++;
+          if (mol->atoms[i].nbonds > maxBondsPerAtom)
+          {
+            maxBondsPerAtom+=5;
+            fremem((void **)&connectivity);
+            goto repeat;
+          }
+          connectivity[maxBondsPerAtom*i+mol->atoms[i].nbonds-1]=j;
+        }
+        else
+        {
+          if (((!strcmp(mol->atoms[i].name, "H") &&
+                 strcmp(mol->atoms[j].name, "H") &&
+                 strcmp(mol->atoms[j].name, "C")) ||
+               (!strcmp(mol->atoms[j].name, "H") &&
+                 strcmp(mol->atoms[i].name, "H") &&
+                 strcmp(mol->atoms[i].name, "C"))) && r1 < hbondThreshold &&
+                 strncmp(mol->atoms[i].element->symbol, "??", 2) &&
+                 strncmp(mol->atoms[j].element->symbol, "??", 2))
           {
             mol->atoms[i].nbonds++;
+            mol->atoms[i].hbonds++;
             if (mol->atoms[i].nbonds > maxBondsPerAtom)
             {
               maxBondsPerAtom+=5;
               fremem((void **)&connectivity);
               goto repeat;
             }
-            connectivity[maxBondsPerAtom*i+mol->atoms[i].nbonds-1]=j;
-          }
-          else
-          {
-            if (((!strcmp(mol->atoms[i].name, "H") &&
-                   strcmp(mol->atoms[j].name, "H") &&
-                   strcmp(mol->atoms[j].name, "C")) ||
-                 (!strcmp(mol->atoms[j].name, "H") &&
-                   strcmp(mol->atoms[i].name, "H") &&
-                   strcmp(mol->atoms[i].name, "C"))) && r1 < hbondThreshold)
-            {
-              mol->atoms[i].nbonds++;
-              mol->atoms[i].hbonds++;
-              if (mol->atoms[i].nbonds > maxBondsPerAtom)
-              {
-                maxBondsPerAtom+=5;
-                fremem((void **)&connectivity);
-                goto repeat;
-              }
-              connectivity[maxBondsPerAtom*i+mol->atoms[i].nbonds-1]=(-j);
-            }
+            connectivity[maxBondsPerAtom*i+mol->atoms[i].nbonds-1]=(-j);
           }
         }
-      } 
-    }
-    else
-    {
-      uc=i;
-      mol->atoms[i].nbonds=3;
-    }
+      }
+    } 
   }
-  if (box == 0.0) box=1.3*sqrt(3.)*mol->atoms[0].rad;
-  box*=1.05/sqrt(3.);
   XtVaGetValues(windows[VIEWER].widget, XtNwidth, &width, XtNheight, &height, NULL);
-  if (setWindow) setWindowSize(box);
   if (mol->bondShift == 0.0)
   {
     pixelToWorld(VIEWER, &xpix, &ypix);
-    mol->bondShift=((double)width*xpix+(double)height*ypix)*0.01;
+    if (lineWidth == 0.0)
+      mol->bondShift=0.005*(double)(width+height)*xpix;
+    else
+      mol->bondShift=lineWidth*xpix;
   }
 
 /* Fill the bonds structure, but check whether user has deleted a particular
@@ -192,7 +168,7 @@ int makeConnectivity(struct MOLECULE *mol, int setWindow, int saveCoord)
           mol->bonds[mol->nb].second=abs(k);
           if (bondType != SINGLE_BONDS)
           {
-            if (mol->atoms[i].nbonds == 1 || mol->atoms[k].nbonds == 1)
+            if (mol->atoms[i].nbonds == 1 || mol->atoms[abs(k)].nbonds == 1)
               bondOrder(mol, &(mol->bonds[mol->nb]), connectivity,
                         maxBondsPerAtom);
             else
@@ -201,12 +177,14 @@ int makeConnectivity(struct MOLECULE *mol, int setWindow, int saveCoord)
           else
             mol->bonds[mol->nb].order=1;
           if (k < 0) mol->bonds[mol->nb].order=(-1);
-          mol->bonds[mol->nb++].frac=mol->atoms[i].rad/bondLength(mol, i, abs(k));
-          if (mol->nb >= mnb)
+	  /* Use the fraction of the bond length corresponding to the Van der Waals
+	     radius of the atom to color a bond according to the atom */
+          mol->bonds[mol->nb].frac=mol->atoms[i].rad/(mol->atoms[i].rad+mol->atoms[abs(k)].rad);
+          if (++mol->nb >= mnb)
           {
             mnb+=20;
             mol->bonds=(struct BOND *)expmem((void *)mol->bonds, mnb,
-                                  sizeof(struct BOND));
+                       sizeof(struct BOND));
           }
         }
       }
@@ -254,7 +232,7 @@ int makeConnectivity(struct MOLECULE *mol, int setWindow, int saveCoord)
         calculateShift(mol, &mol->bonds[mol->nb], connectivity, maxBondsPerAtom,
                        mol->addedBonds[3*i], mol->addedBonds[3*i+1]);
       mol->bonds[mol->nb++].frac=mol->atoms[mol->addedBonds[3*i]].rad
-                                /bondLength(mol, mol->addedBonds[3*i],
+                                /bondLength(mol->atoms, mol->addedBonds[3*i],
                                 mol->addedBonds[3*i+1]);
       if (mol->nb >= mnb)
       {
@@ -288,40 +266,36 @@ int makeConnectivity(struct MOLECULE *mol, int setWindow, int saveCoord)
     mol->bonds=(struct BOND *)expmem((void *)mol->bonds,
                           mol->nb, sizeof(struct BOND));
 
-/* Add unit cell edges as "bonds" */
+/* Add unit cell edges */
 
-  if (uc)
+  if (mol->unitcell)
   {
-    mol->bonds=(struct BOND *)expmem((void *)mol->bonds, mol->nb+12, sizeof(struct BOND));
-    mol->bonds[mol->nb].first=uc-7;
-    mol->bonds[mol->nb++].second=uc-6;
-    mol->bonds[mol->nb].first=uc-7;
-    mol->bonds[mol->nb++].second=uc-5;
-    mol->bonds[mol->nb].first=uc-7;
-    mol->bonds[mol->nb++].second=uc-4;
-    mol->bonds[mol->nb].first=uc-6;
-    mol->bonds[mol->nb++].second=uc-3;
-    mol->bonds[mol->nb].first=uc-6;
-    mol->bonds[mol->nb++].second=uc;
-    mol->bonds[mol->nb].first=uc-5;
-    mol->bonds[mol->nb++].second=uc-1;
-    mol->bonds[mol->nb].first=uc-5;
-    mol->bonds[mol->nb++].second=uc;
-    mol->bonds[mol->nb].first=uc-4;
-    mol->bonds[mol->nb++].second=uc-3;
-    mol->bonds[mol->nb].first=uc-4;
-    mol->bonds[mol->nb++].second=uc-1;
-    mol->bonds[mol->nb].first=uc-3;
-    mol->bonds[mol->nb++].second=uc-2;
-    mol->bonds[mol->nb].first=uc-2;
-    mol->bonds[mol->nb++].second=uc-1;
-    mol->bonds[mol->nb].first=uc-2;
-    mol->bonds[mol->nb++].second=uc;
-    for (i=mol->nb-12; i<mol->nb; i++)
-    {
-      mol->bonds[i].order=1;
-      mol->bonds[i].frac=0.5;
-    }
+    mol->unitcell->edges=(struct EDGE *)getmem(12, sizeof(struct EDGE));
+    mol->unitcell->ne=0;
+    mol->unitcell->edges[mol->unitcell->ne].first=0;
+    mol->unitcell->edges[mol->unitcell->ne++].second=1;
+    mol->unitcell->edges[mol->unitcell->ne].first=0;
+    mol->unitcell->edges[mol->unitcell->ne++].second=2;
+    mol->unitcell->edges[mol->unitcell->ne].first=0;
+    mol->unitcell->edges[mol->unitcell->ne++].second=3;
+    mol->unitcell->edges[mol->unitcell->ne].first=1;
+    mol->unitcell->edges[mol->unitcell->ne++].second=4;
+    mol->unitcell->edges[mol->unitcell->ne].first=1;
+    mol->unitcell->edges[mol->unitcell->ne++].second=7;
+    mol->unitcell->edges[mol->unitcell->ne].first=2;
+    mol->unitcell->edges[mol->unitcell->ne++].second=6;
+    mol->unitcell->edges[mol->unitcell->ne].first=2;
+    mol->unitcell->edges[mol->unitcell->ne++].second=7;
+    mol->unitcell->edges[mol->unitcell->ne].first=3;
+    mol->unitcell->edges[mol->unitcell->ne++].second=4;
+    mol->unitcell->edges[mol->unitcell->ne].first=3;
+    mol->unitcell->edges[mol->unitcell->ne++].second=6;
+    mol->unitcell->edges[mol->unitcell->ne].first=4;
+    mol->unitcell->edges[mol->unitcell->ne++].second=5;
+    mol->unitcell->edges[mol->unitcell->ne].first=5;
+    mol->unitcell->edges[mol->unitcell->ne++].second=6;
+    mol->unitcell->edges[mol->unitcell->ne].first=5;
+    mol->unitcell->edges[mol->unitcell->ne++].second=7;
   }
   if (debug == 1)
   {
@@ -337,10 +311,74 @@ int makeConnectivity(struct MOLECULE *mol, int setWindow, int saveCoord)
       else
         printf("\n");
     }
+    if (mol->unitcell)
+    {
+      printf("Unit cell edges:\n");
+      for (i=0; i<mol->unitcell->ne; i++)
+        printf("%d: %d-%d\n", i, mol->bonds[i].first+1, mol->bonds[i].second+1);
+    }
   }
   return(TRUE);
 }
 
+double centerMolecule(struct MOLECULE *mol)
+{
+  double mass, box, r;
+  int i;
+
+  /* Translate molecule so that it is centered at its
+     center of gravity and determine the size of the cube
+     enclosing it taking into account the van der Waals
+     radii of the atoms */
+  mol->transx=0.0;
+  mol->transy=0.0;
+  mol->transz=0.0;
+  mass=0.0;
+  for (i=0; i<mol->na; i++)
+  {
+    mol->transx+=mol->atoms[i].mass*mol->atoms[i].x;
+    mol->transy+=mol->atoms[i].mass*mol->atoms[i].y;
+    mol->transz+=mol->atoms[i].mass*mol->atoms[i].z;
+    mass+=mol->atoms[i].mass;
+  }
+  mol->transx=-mol->transx/mass;
+  mol->transy=-mol->transy/mass;
+  mol->transz=-mol->transz/mass;
+  if (debug == 1)
+    printf("Shifting molecule by %f %f %f.\n", mol->transx, mol->transy,
+           mol->transz);
+  box=0.0;
+  for (i=0; i<mol->na; i++)
+  {
+    mol->atoms[i].x+=mol->transx;
+    mol->atoms[i].y+=mol->transy;
+    mol->atoms[i].z+=mol->transz;
+    r=mol->atoms[i].radScale*mol->atoms[i].rad;
+    box=fabs(mol->atoms[i].x)+r > box ? fabs(mol->atoms[i].x)+r : box;
+    box=fabs(mol->atoms[i].y)+r > box ? fabs(mol->atoms[i].y)+r : box;
+    box=fabs(mol->atoms[i].z)+r > box ? fabs(mol->atoms[i].z)+r : box;
+  }
+  if (mol->unitcell)
+  {
+    for (i=0; i<mol->unitcell->nc; i++)
+    {
+      mol->unitcell->corners[i].x+=mol->transx;
+      mol->unitcell->corners[i].y+=mol->transy;
+      mol->unitcell->corners[i].z+=mol->transz;
+      r=mol->unitcell->corners[i].radScale*mol->unitcell->corners[i].rad;
+      box=fabs(mol->unitcell->corners[i].x)+r > box ? fabs(mol->unitcell->corners[i].x)+r : box;
+      box=fabs(mol->unitcell->corners[i].y)+r > box ? fabs(mol->unitcell->corners[i].y)+r : box;
+      box=fabs(mol->unitcell->corners[i].z)+r > box ? fabs(mol->unitcell->corners[i].z)+r : box;
+    }
+  }
+  if (box == 0.0) box=mol->atoms[0].rad;
+  /* Add 5 % space and make it the space diagonal of the cube
+     so that rotations do not move any atom out of the clipping
+     volume */
+  box*=1.05*sqrt(3.);
+  return(box);
+}
+
 void setBondType(Widget widget, caddr_t which, XmToggleButtonCallbackStruct *data)
 {
   register int i;
