@@ -4,12 +4,15 @@
 *                                                                              *
 *                       M O L E C U L E M O D U L E . C                        *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 2000                   *
+*                 Copyright (c) Joerg-R. Hill, October 2003                    *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: moleculemodule.c,v 1.1 2000/12/10 15:12:50 jrh Exp $
+* $Id: moleculemodule.c,v 1.2 2003/11/07 11:07:39 jrh Exp $
 * $Log: moleculemodule.c,v $
+* Revision 1.2  2003/11/07 11:07:39  jrh
+* Release 2.4
+*
 * Revision 1.1  2000/12/10 15:12:50  jrh
 * Initial revision
 *
@@ -22,6 +25,11 @@
 #define PyMolecule_API_pointers 1
 #define PyMoleculeSpec_Type_NUM 0
 
+#define ENTHALPY      1L
+#define ENTROPY       2L
+#define GIBBS_ENERGY  3L
+#define HEAT_CAPACITY 4L
+
 PyMoleculeSpecObject *molecule_new(void);
 static PyObject *molecule_translate(PyObject *, PyObject *);
 static PyObject *molecule_rotate(PyObject *, PyObject *);
@@ -32,6 +40,7 @@ static PyObject *molecule_showForces(PyObject *, PyObject *);
 static PyObject *molecule_getAtoms(PyObject *, PyObject *);
 static PyObject *molecule_getBonds(PyObject *, PyObject *);
 static PyObject *molecule_getWavenumbers(PyObject *, PyObject *);
+static PyObject *molecule_getMOEnergies(PyObject *, PyObject *);
 static PyObject *molecule_title(PyObject *, PyObject *);
 static PyObject *molecule_bondAverage(PyObject *, PyObject *);
 static PyObject *molecule_bondLength(PyObject *, PyObject *);
@@ -40,6 +49,7 @@ static PyObject *molecule_torsionAngle(PyObject *, PyObject *);
 static PyObject *molecule_getThermodynamics(PyObject *, PyObject *);
 static PyObject *molecule_reaction(PyObject *, PyObject *);
 static PyObject *molecule_showElectrons(PyObject *, PyObject *);
+static PyObject *molecule_showGrid(PyObject *, PyObject *);
 static PyObject *molecule_selectBasisfunction(PyObject *, PyObject *);
 static PyObject *molecule_unitCell(PyObject *, PyObject *);
 static PyObject *molecule_millerPlane(PyObject *, PyObject *);
@@ -64,9 +74,9 @@ extern PySpectrumSpecObject *spectrum_new(void);
 extern PyHistorySpecObject *history_new(void);
 extern PyEnergyLevelSpecObject *energylevels_new(void);
 extern double bondAverage(struct MOLECULE *, int);
-extern double bondLength(struct MOLECULE *, int, int);
-extern double bondAngle(struct MOLECULE *, int, int, int);
-extern double torsionAngle(struct MOLECULE *, int, int, int, int);
+extern double bondLength(struct ATOM *, int, int);
+extern double bondAngle(struct ATOM *, int, int, int);
+extern double torsionAngle(struct ATOM *, int, int, int, int);
 extern void *getmem(size_t, size_t);
 extern void *expmem(void *, size_t, size_t);
 extern void fremem(void **);
@@ -75,11 +85,11 @@ extern void calcmo(void);
 extern int  existsGridObject(void);
 extern void deleteGridObjects(void);
 extern void redraw(int);
-extern char *bfname(int);
+/* extern char *bfname(int);*/
 extern void bfnamn(int, int *, int*);
 extern void annotateWavefunction(void);
 extern void buildMolecule(struct MOLECULE *, int, int, int);
-extern void expandCell(void);
+extern void expandCell(int);
 extern void millerPlane(void);
 extern int checkInterrupt(void);
 extern void newMolecule(Widget, caddr_t, XmAnyCallbackStruct *);
@@ -133,6 +143,7 @@ static PyMethodDef molecule_methods[] = {
   {"getAtoms",            molecule_getAtoms, 1},
   {"getBonds",            molecule_getBonds, 1},
   {"getWavenumbers",      molecule_getWavenumbers, 1},
+  {"getMOEnergies",       molecule_getMOEnergies, 1},
   {"title",               molecule_title, 1},
   {"bondAverage",         molecule_bondAverage, 1},
   {"bondLength",          molecule_bondLength, 1},
@@ -141,6 +152,7 @@ static PyMethodDef molecule_methods[] = {
   {"getThermodynamics",   molecule_getThermodynamics, 1},
   {"reaction",            molecule_reaction, 1},
   {"showElectrons",       molecule_showElectrons, 1},
+  {"showGrid",            molecule_showGrid, 1},
   {"selectBasisfunction", molecule_selectBasisfunction, 1},
   {"unitCell",            molecule_unitCell, 1},
   {"millerPlane",         molecule_millerPlane, 1},
@@ -258,7 +270,7 @@ static PyObject *molecule_getHistory(PyObject *self, PyObject *args)
   }
   s=(PyMoleculeSpecObject *)self;
   which=s->moleculeID;
-  if (molecules[which].nhist == 0)
+  if (molecules[which].nhist < 2)
   {
     Py_INCREF(Py_None);
     return(Py_None);
@@ -291,7 +303,7 @@ static PyObject *molecule_getAtoms(PyObject *self, PyObject *args)
   PyMoleculeSpecObject *s;
   PyAtomSpecObject *atom;
   PyObject *list;
-  int which, na, i;
+  int which, i;
 
   if (checkInterrupt()) return(NULL);
   if (!self)
@@ -301,22 +313,15 @@ static PyObject *molecule_getAtoms(PyObject *self, PyObject *args)
   }
   s=(PyMoleculeSpecObject *)self;
   which=s->moleculeID;
-  if (molecules[which].existsUnitCell)
-    na=molecules[which].na-8;
-  else
-    na=molecules[which].na;
-  if ((list=PyList_New(na)))
+  if ((list=PyList_New(molecules[which].na)))
   {
     for (i=0; i<molecules[which].na; i++)
     {
-	if (strncmp(molecules[which].atoms[i].name, "Uc", 2))
-	{
-        atom=atom_new();
-        if (atom == NULL) continue;
-        atom->moleculeID=which;
-        atom->atomID=i;
-        PyList_SetItem(list, i, (PyObject *)atom);
-      }
+      atom=atom_new();
+      if (atom == NULL) continue;
+      atom->moleculeID=which;
+      atom->atomID=i;
+      PyList_SetItem(list, i, (PyObject *)atom);
     }
     return((PyObject *)list);
   }
@@ -389,6 +394,42 @@ static PyObject *molecule_getWavenumbers(PyObject *self, PyObject *args)
   return((PyObject *)list);
 }
 
+static PyObject *molecule_getMOEnergies(PyObject *self, PyObject *args)
+{
+  PyMoleculeSpecObject *s;
+  PyObject *list, *tuple;
+  int which, i;
+
+  if (checkInterrupt()) return(NULL);
+  if (!self)
+  {
+    Py_INCREF(Py_None);
+    return(Py_None);
+  }
+  s=(PyMoleculeSpecObject *)self;
+  which=s->moleculeID;
+  if (molecules[which].orbitals == NULL)
+  {
+    Py_INCREF(Py_None);
+    return(Py_None);
+  }
+  if ((list=PyList_New(molecules[which].nbasfu)))
+  {
+    for (i=0; i<molecules[which].nbasfu; i++)
+    {
+      if ((tuple=PyTuple_New(4)))
+      {
+        PyTuple_SetItem(tuple, 0, PyFloat_FromDouble(molecules[which].orbitals[i].energy));
+        PyTuple_SetItem(tuple, 1, PyFloat_FromDouble(molecules[which].orbitals[i].occupation));
+        PyTuple_SetItem(tuple, 2, PyInt_FromLong((long)(molecules[which].orbitals[i].spin)));
+        PyTuple_SetItem(tuple, 3, PyString_FromString(molecules[which].orbitals[i].symmetry));
+        PyList_SetItem(list, i, tuple);
+      }
+    }
+  }
+  return((PyObject *)list);
+}
+
 static PyObject *molecule_title(PyObject *self, PyObject *args)
 {
   PyMoleculeSpecObject *s;
@@ -475,7 +516,7 @@ static PyObject *molecule_bondLength(PyObject *self, PyObject *args)
     j=atom2->atomID;
     if (unit == NULL) /* get */
     {
-      r=bondLength(&molecules[which], i, j);
+      r=bondLength((&molecules[which])->atoms, i, j);
       return(PyFloat_FromDouble(r));
     }
     else              /* set */
@@ -519,7 +560,7 @@ static PyObject *molecule_bondAngle(PyObject *self, PyObject *args)
     k=atom3->atomID;
     if (r == 400.0e0) /* get */
     {
-      r=bondAngle(&molecules[which], i, j, k);
+      r=bondAngle((&molecules[which])->atoms, i, j, k);
       return(PyFloat_FromDouble(r));
     }
     else              /* set */
@@ -565,7 +606,7 @@ static PyObject *molecule_torsionAngle(PyObject *self, PyObject *args)
     l=atom4->atomID;
     if (r == 400.e0) /* get */
     {
-      r=torsionAngle(&molecules[which], i, j, k, l);
+      r=torsionAngle((&molecules[which])->atoms, i, j, k, l);
       return(PyFloat_FromDouble(r));
     }
     else             /* set */
@@ -598,61 +639,67 @@ static PyObject *molecule_getThermodynamics(PyObject *self, PyObject *args)
   mol=&molecules[which];
   switch (quantity)
   {
-    case 1: switch (type)
-            {
-		  case TRANSLATION: value=0.001*mol->htrl;
-		                    break;
-		  case PV:          value=0.001*mol->pv;
-		                    break;
-		  case ROTATION:    value=0.001*mol->hrot;
-		                    break;
-		  case VIBRATION:   value=0.001*mol->hvib;
-		                    break;
-		  case TOTAL:       value=0.001*(mol->htrl+mol->pv+mol->hrot+mol->hvib);
-		                    break;
-            }
-		break;
-    case 2: switch (type)
-            {
-		  case TRANSLATION: value=mol->strl;
-		                    break;
-		  case PV:          PyErr_SetString(PyExc_ValueError, "pV is invalid for entropy");
-		                    return(NULL);
-					  /* NOTREACHED */
-		                    break;
-		  case ROTATION:    value=mol->srot;
-		                    break;
-		  case VIBRATION:   value=mol->svib;
-		                    break;
-		  case TOTAL:       value=mol->strl+mol->srot+mol->svib;
-		                    break;
-            }
-		break;
-    case 3: switch (type)
-            {
-		  case TRANSLATION: value=0.001*(mol->htrl-temp*mol->strl);
-		                    break;
-		  case PV:          value=0.001*mol->pv;
-		                    break;
-		  case ROTATION:    value=0.001*(mol->hrot-temp*mol->srot);
-		                    break;
-		  case VIBRATION:   value=0.001*(mol->hvib-temp*mol->svib);
-		                    break;
-		  case TOTAL:       value=0.001*(mol->htrl+mol->pv+mol->hrot
-		                         +mol->hvib-temp*(mol->strl+mol->srot+mol->svib));
-		                    break;
-            }
-    case 4: switch (type)
-            {
-		  case TRANSLATION: value=mol->ctrl;
-		                    break;
-		  case ROTATION:    value=mol->crot;
-		                    break;
-		  case VIBRATION:   value=mol->cvib;
-		                    break;
-		  case TOTAL:       value=mol->ctrl+mol->crot+mol->cvib;
-		                    break;
-            }
+    case ENTHALPY:      switch (type)
+                        {
+                          case TRANSLATION: value=0.001*mol->htrl;
+                                            break;
+                          case PV:          value=0.001*mol->pv;
+                                            break;
+                          case ROTATION:    value=0.001*mol->hrot;
+                                            break;
+                          case VIBRATION:   value=0.001*mol->hvib;
+                                            break;
+                          case TOTAL:       value=0.001*(mol->htrl+mol->pv+mol->hrot+mol->hvib);
+                                            break;
+                        }
+                        break;
+    case ENTROPY:       switch (type)
+                        {
+                          case TRANSLATION: value=mol->strl;
+                                            break;
+                          case PV:          PyErr_SetString(PyExc_ValueError, "pV is invalid for entropy");
+                                            return(NULL);
+                                            /* NOTREACHED */
+                                            break;
+                          case ROTATION:    value=mol->srot;
+                                            break;
+                          case VIBRATION:   value=mol->svib;
+                                            break;
+                          case TOTAL:       value=mol->strl+mol->srot+mol->svib;
+                                            break;
+                        }
+                        break;
+    case GIBBS_ENERGY:  switch (type)
+                        {
+                          case TRANSLATION: value=0.001*(mol->htrl-temp*mol->strl);
+                                            break;
+                          case PV:          value=0.001*mol->pv;
+                                            break;
+                          case ROTATION:    value=0.001*(mol->hrot-temp*mol->srot);
+                                            break;
+                          case VIBRATION:   value=0.001*(mol->hvib-temp*mol->svib);
+                                            break;
+                          case TOTAL:       value=0.001*(mol->htrl+mol->pv+mol->hrot
+                                                 +mol->hvib-temp*(mol->strl+mol->srot+mol->svib));
+                                            break;
+                        }
+                        break;
+    case HEAT_CAPACITY: switch (type)
+                        {
+                          case TRANSLATION: value=mol->ctrl;
+                                            break;
+                          case PV:          PyErr_SetString(PyExc_ValueError, "pV is invalid for heat capacity");
+                                            return(NULL);
+                                            /* NOTREACHED */
+                                            break;
+                          case ROTATION:    value=mol->crot;
+                                            break;
+                          case VIBRATION:   value=mol->cvib;
+                                            break;
+                          case TOTAL:       value=mol->ctrl+mol->crot+mol->cvib;
+                                            break;
+                        }
+	                break;
   }
   return(PyFloat_FromDouble(value));
 }
@@ -704,33 +751,66 @@ static PyObject *molecule_showElectrons(PyObject *self, PyObject *args)
   switch (iwavef)
   {
     case FALSE:             deleteGridObjects();
-				    break;
+                            break;
     case BASIS_FUNCTION:
     case BASIS_IN_MO:
     case MOLECULAR_ORBITAL:
     case DENSITY:           if (!existsGridObject())
                             {
-				      calcmo();
-					annotateWavefunction();
+                              calcmo();
+                              annotateWavefunction();
                             }
-				    break;
+                            break;
     default:                PyErr_SetString(PyExc_ValueError,
-							  "Unknown type of wave function");
+                                            "Unknown type of wave function");
                             return(NULL);
-				    /* NOTREACHED */
-				    break;
+                            /* NOTREACHED */
+                            break;
   }
   redraw(VIEWER);
   Py_INCREF(Py_None);
   return(Py_None);
 }
 
+static PyObject *molecule_showGrid(PyObject *self, PyObject *args)
+{
+  PyMoleculeSpecObject *s;
+  struct MOLECULE *mol;
+  int which, igrid;
+
+  if (checkInterrupt()) return(NULL);
+  if (!self)
+  {
+    Py_INCREF(Py_None);
+    return(Py_None);
+  }
+
+  s=(PyMoleculeSpecObject *)self;
+  which=s->moleculeID;
+  if (!PyArg_ParseTuple(args, "i|i", &igrid, &interp)) return NULL;
+  mol=&molecules[which];
+  if (mol->gridObjects == (struct GRIDOBJECT *)NULL)
+  {
+    PyErr_SetString(PyExc_ValueError, "No grids have been read for this molecule");
+    return(NULL);
+  }
+  if (igrid < 1 || igrid > mol->ngridobjects)
+  {
+    PyErr_SetString(PyExc_ValueError, "A grid with this number does not exist");
+    return(NULL);
+  }
+  iwavef=DENSITY+igrid;
+  redraw(VIEWER);
+  Py_INCREF(Py_None);
+  return(Py_None);
+}
+
 static PyObject *molecule_selectBasisfunction(PyObject *self, PyObject *args)
 {
   PyMoleculeSpecObject *s;
   PyAtomSpecObject *atom;
   struct MOLECULE *mol;
-  int which, count, start, end;
+  int which, count;
   char *name;
   register int i, found=FALSE;
 
@@ -755,18 +835,17 @@ static PyObject *molecule_selectBasisfunction(PyObject *self, PyObject *args)
     return(NULL);
   }
   mol=&molecules[which];
-  bfnamn(atom->atomID, &start, &end);
-  for (i=start; i<=end; i++)
+  for (i=0; i<mol->atoms[atom->atomID].nBasisFunctions; i++)
   {
-    if (!strcmp(bfname(i), name))
+    if (!strcmp(mol->atoms[atom->atomID].basisFunctionNames[i], name))
     {
       count--;
-	if (count == 0)
-	{
-	  mol->ibasfu=i;
-	  found=TRUE;
-	  break;
-	}
+      if (count == 0)
+      {
+        mol->ibasfu=i;
+        found=TRUE;
+        break;
+      }
     }
   }
   if (!found)
@@ -798,35 +877,32 @@ static PyObject *molecule_unitCell(PyObject *self, PyObject *args)
   s=(PyMoleculeSpecObject *)self;
   which=s->moleculeID;
   mol=&molecules[which];
-  xfac=mol->cellFactor[0];
-  yfac=mol->cellFactor[1];
-  zfac=mol->cellFactor[2];
-  if (!PyArg_ParseTuple(args, "i|ddd", &visible, &xfac, &yfac, &zfac))
-    return NULL;
-  if (mol->existsUnitCell == 0)
+  if (mol->unitcell == 0)
   {
     PyErr_SetString(PyExc_ValueError, "No unit cell defined");
     return(NULL);
   }
-  else
+  xfac=mol->unitcell->factor[0];
+  yfac=mol->unitcell->factor[1];
+  zfac=mol->unitcell->factor[2];
+  if (!PyArg_ParseTuple(args, "i|ddd", &visible, &xfac, &yfac, &zfac))
+    return NULL;
+  if (visible != TRUE && visible != FALSE)
   {
-    if (visible != TRUE && visible != FALSE)
-    {
-      PyErr_SetString(PyExc_ValueError, "Visible must be ON or OFF");
-      return(NULL);
-    }
-    showUnitCell=visible;
-    if (xfac != mol->cellFactor[0] || yfac != mol->cellFactor[1] ||
-        zfac != mol->cellFactor[2])
-    {
-       mol->cellFactor[0]=xfac;
-       mol->cellFactor[1]=yfac;
-       mol->cellFactor[2]=zfac;
-       expandCell();
-    }
-    Py_INCREF(Py_None);
-    return(Py_None);
+    PyErr_SetString(PyExc_ValueError, "Visible must be ON or OFF");
+    return(NULL);
   }
+  showUnitCell=visible;
+  if (xfac != mol->unitcell->factor[0] || yfac != mol->unitcell->factor[1] ||
+      zfac != mol->unitcell->factor[2])
+  {
+     mol->unitcell->factor[0]=xfac;
+     mol->unitcell->factor[1]=yfac;
+     mol->unitcell->factor[2]=zfac;
+     expandCell(TRUE);
+  }
+  Py_INCREF(Py_None);
+  return(Py_None);
 }
 
 static PyObject *molecule_millerPlane(PyObject *self, PyObject *args)
@@ -845,24 +921,24 @@ static PyObject *molecule_millerPlane(PyObject *self, PyObject *args)
   s=(PyMoleculeSpecObject *)self;
   which=s->moleculeID;
   mol=&molecules[which];
-  h=mol->miller[0];
-  k=mol->miller[1];
-  l=mol->miller[2];
-  if (!PyArg_ParseTuple(args, "i|iii", &visible, &h, &k, &l)) return NULL;
-  if (mol->existsUnitCell == 0)
+  if (mol->unitcell == 0)
   {
     PyErr_SetString(PyExc_ValueError, "No unit cell defined");
     return(NULL);
   }
+  h=mol->unitcell->miller[0];
+  k=mol->unitcell->miller[1];
+  l=mol->unitcell->miller[2];
+  if (!PyArg_ParseTuple(args, "i|iii", &visible, &h, &k, &l)) return NULL;
   if (visible != TRUE && visible != FALSE)
   {
     PyErr_SetString(PyExc_ValueError, "Visible must be ON or OFF");
     return(NULL);
   }
-  mol->miller[0]=h;
-  mol->miller[1]=k;
-  mol->miller[2]=l;
-  mol->showMiller=visible;
+  mol->unitcell->miller[0]=h;
+  mol->unitcell->miller[1]=k;
+  mol->unitcell->miller[2]=l;
+  mol->unitcell->showMiller=visible;
   millerPlane();
   Py_INCREF(Py_None);
   return(Py_None);
@@ -895,7 +971,7 @@ static PyObject *molecule_addAtom(PyObject *self, PyObject *args)
     if (!strncmp(elements[i].symbol, symbol, 2))
     {
       element=i;
-	found=TRUE;
+      found=TRUE;
     }
   }
   if (!found)
@@ -959,10 +1035,10 @@ void initMoleculeModule(void)
   PyDict_SetItemString(dict, "TRIPLE", PyInt_FromLong(3L));
   PyDict_SetItemString(dict, "CONJUGATED", PyInt_FromLong(-2L));
   PyDict_SetItemString(dict, "HYDROGENBOND", PyInt_FromLong(-1L));
-  PyDict_SetItemString(dict, "ENTHALPY", PyInt_FromLong(1L));
-  PyDict_SetItemString(dict, "ENTROPY", PyInt_FromLong(2L));
-  PyDict_SetItemString(dict, "GIBBS_ENERGY", PyInt_FromLong(3L));
-  PyDict_SetItemString(dict, "HEAT_CAPACITY", PyInt_FromLong(4L));
+  PyDict_SetItemString(dict, "ENTHALPY", PyInt_FromLong(ENTHALPY));
+  PyDict_SetItemString(dict, "ENTROPY", PyInt_FromLong(ENTROPY));
+  PyDict_SetItemString(dict, "GIBBS_ENERGY", PyInt_FromLong(GIBBS_ENERGY));
+  PyDict_SetItemString(dict, "HEAT_CAPACITY", PyInt_FromLong(HEAT_CAPACITY));
   PyDict_SetItemString(dict, "TRANSLATION", PyInt_FromLong((long)TRANSLATION));
   PyDict_SetItemString(dict, "PV", PyInt_FromLong((long)PV));
   PyDict_SetItemString(dict, "ROTATION", PyInt_FromLong((long)ROTATION));
@@ -978,6 +1054,9 @@ void initMoleculeModule(void)
   PyDict_SetItemString(dict, "REACTANT", PyInt_FromLong((long)REACTANT));
   PyDict_SetItemString(dict, "PRODUCT", PyInt_FromLong((long)PRODUCT));
   PyDict_SetItemString(dict, "ALLREACTIONS", PyInt_FromLong((long)ALLREACTIONS));
+  PyDict_SetItemString(dict, "ALPHA+BETA", PyInt_FromLong((long)ALPHAANDBETA));
+  PyDict_SetItemString(dict, "ALPHA", PyInt_FromLong((long)ALPHA));
+  PyDict_SetItemString(dict, "BETA", PyInt_FromLong((long)BETA));
 }
 
 void setInternal(struct MOLECULE *mol, int type, int i, int j, int k, int l, char *value)
