@@ -4,12 +4,15 @@
 *                                                                              *
 *                          M O U S E A C T I O N . C                           *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 2000                   *
+*                 Copyright (c) Joerg-R. Hill, October 2003                    *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: mouseaction.c,v 1.5 2000/12/10 15:13:02 jrh Exp $
+* $Id: mouseaction.c,v 1.6 2003/11/07 11:07:48 jrh Exp $
 * $Log: mouseaction.c,v $
+* Revision 1.6  2003/11/07 11:07:48  jrh
+* Release 2.4
+*
 * Revision 1.5  2000/12/10 15:13:02  jrh
 * Release 2.3
 *
@@ -58,16 +61,15 @@ void setAnimation(int);
 void selectMolecule(Widget, caddr_t, XmToggleButtonCallbackStruct *);
 void setWindow(int);
 void deleteSelection(void);
+void trim(char *);
 
 extern void drawMolecule(Widget, caddr_t, XmDrawingAreaCallbackStruct *);
 extern void redraw(int);
 extern void drawSpectrum(Widget, caddr_t, XmDrawingAreaCallbackStruct *);
 extern void drawMODiagram(Widget, caddr_t, XmDrawingAreaCallbackStruct *);
 extern void processInput(Widget, caddr_t, XmDrawingAreaCallbackStruct *);
-extern void calcIntern(int);
 extern Boolean normalMode(XtPointer);
 extern Boolean animateHistory(XtPointer);
-extern char *bfname(int);
 extern Widget makeMenu(Widget, int, char *, struct MenuItem *);
 extern void makeINSIntensity(void);
 extern char *getStringResource(Widget, char *);
@@ -100,8 +102,12 @@ extern Widget makeViewerMenu(Widget);
 extern int checkZoom(int);
 extern void setMenu(struct MOLECULE *);
 extern void updateMOs(int);
-extern void destroyAnnotation(void);
-extern void expandCell(void);
+extern void expandCell(int);
+extern void getScreenCoordinates(double, double, double, double *, double *,
+                                 double *);
+extern void getWorldCoordinates(double, double, double, double *, double *,
+                                double *);
+extern void annotateNormalMode(struct MOLECULE *);
 
 extern XtAppContext app;
 extern struct WINDOW windows[];
@@ -113,7 +119,7 @@ extern int picking, swapBuffers, setins, projectionMode;
 extern int bfatom, annotate;
 extern int rotateXY, rotateZ, moveItem;
 extern int debug;
-extern int nmolecule;
+extern int nmolecule, nAnnotations;
 extern int historyAnnotation, normalModeAnnotation;
 extern int modifyMolecule, bondOrderDefault;
 extern XtAppContext app;
@@ -130,7 +136,9 @@ void viewerMouseAction(XEvent *event, int state)
   static struct PushButtonRow buttons[] = {{"ok", GetMessageBoxButton,
                                             (XtPointer)0, NULL}};
   struct MOLECULE *mol;
+  Dimension width, height;
   GLint picked;
+  double xs, ys, zs, xpix, ypix;
   float black[4]={0.0, 0.0, 0.0, 0.0};
   static int new=TRUE, moveItemSave;
   static int molclicked[4], ic=(-1);
@@ -173,8 +181,8 @@ void viewerMouseAction(XEvent *event, int state)
       {
         if (!(event->xmotion.state & ShiftMask))
         {
-          setupProjection(picking);
-          modifyAnnotation((picked & 0x3fffffff), event->xbutton.x, event->xbutton.y);
+	  l=picked & 0x3fffffff;
+          modifyAnnotation(l, event->xbutton.x, event->xbutton.y);
         }
         else
         {
@@ -183,7 +191,7 @@ void viewerMouseAction(XEvent *event, int state)
         }
         return;
       }
-      if (picked & 0x40000000)
+      if (picked & 0x40000000 && imol != 0x80000000)
       {
         l=picked & 0x3fffffff;
         switch (molecules[imol].internals[l].type)
@@ -196,26 +204,42 @@ void viewerMouseAction(XEvent *event, int state)
           case TORSION:     form=formTorsionAngle;
                             break;
         }
-        if (annotation == NULL) ic=(-1);
-        if (ic >= 0)
+	ic=(-1);
+	for (i=0; i<nAnnotations; i++)
+	{
+	  if (annotation[i].userdata == l)
+	  {
+	    ic=i;
+	    break;
+	  }
+	}
+        if (ic != -1)
         {
-          i=annotation[ic].userdata;
-          molecules[imol].internals[i].type=(-molecules[imol].internals[i].type);
+          molecules[imol].internals[l].type=(-molecules[imol].internals[l].type);
           deleteAnnotation(&ic);
-          destroyAnnotation();
         }
         sprintf(line, form, molecules[imol].internals[l].value);
-        ic=makeAnnotation(ic, COORDINATES, (float)molecules[imol].internals[l].x,
-                          (float)molecules[imol].internals[l].y, (float)molecules[imol].internals[l].z,
-                          stdcol[BLACK], black, GEOMETRY | EDITABLE, l, line);
+        trim(line);
         glXMakeCurrent(XtDisplay(windows[VIEWER].widget), XtWindow(windows[VIEWER].widget),
-                                  windows[VIEWER].context);
+                                 windows[VIEWER].context);
         setupProjection(picking);
         setupModelview(imol);
+	getScreenCoordinates(molecules[imol].internals[l].x, molecules[imol].internals[l].y, 
+                             molecules[imol].internals[l].z, &xs, &ys, &zs);
+        XtVaGetValues(windows[VIEWER].widget, XtNwidth, &width, XtNheight, &height, NULL);
+	pixelToWorld(VIEWER, &xpix, &ypix);
+        ic=makeAnnotation((-1), COORDINATES, (float)(xpix*(xs-0.5*(double)width)), (float)(ypix*(0.5*(double)height-ys)),
+  	                  (float)zs, stdcol[BLACK], black, GEOMETRY | EDITABLE, l, line);
         modifyAnnotation(ic, event->xbutton.x, event->xbutton.y);
         molecules[imol].internals[l].type=(-molecules[imol].internals[l].type);
         return;
       }
+      if (picked & 0x20000000)
+      {
+        l=(picked & 0x1fffffff);
+        molecules[imol].unitcell[0].corners[l].flags^=SELECTED;
+/*      return; */
+      }
       if (!(event->xmotion.state & ShiftMask))
       {
         if (new)
@@ -223,7 +247,8 @@ void viewerMouseAction(XEvent *event, int state)
           new=FALSE;
           intcoord=0;
         }
-	  molecules[imol].atoms[picked].flags^=SELECTED;
+	  if (!(picked & 0x20000000))
+	    molecules[imol].atoms[picked].flags^=SELECTED;
         /* check whether atom has already been clicked on */
         j=FALSE;
         for (i=0; i<intcoord; i++)
@@ -427,8 +452,8 @@ void viewerMouseAction(XEvent *event, int state)
                     break;
     case Button3:   if (state == TRUE)
                     {
-			    if (!(event->xmotion.state & ShiftMask))
-			    {
+                      if (!(event->xmotion.state & ShiftMask))
+                      {
                         new=TRUE;
                         if (windows[VIEWER].set >= 0)
                           mol=&molecules[windows[VIEWER].set];
@@ -578,11 +603,11 @@ void clearGeometry(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
 
 void spectrumMouseAction(XmDrawingAreaCallbackStruct *data, int state)
 {
-  const GLfloat black[4] = {0.0, 0.0, 0.0, 0.0};
+/*const GLfloat black[4] = {0.0, 0.0, 0.0, 0.0};*/
   struct MOLECULE *mol;
   static struct PushButtonRow button[] = {{"continue", GetMessageBoxButton, (XtPointer)0, NULL}};
   int mode, imol;
-  char line[MAXLENLINE], *word;
+  char /*line[MAXLENLINE],*/ *word;
   register int i;
 
   mol=&molecules[windows[SPECTRUM].set];
@@ -612,11 +637,7 @@ void spectrumMouseAction(XmDrawingAreaCallbackStruct *data, int state)
           restoreGeometry(mol->coord, windows[SPECTRUM].set);
           if (animate == DISTORT) distortGeometry(amplitude);
         }
-        word=getStringResource(topShell, "wavenumberTitle");
-        sprintf(line, word, mol->normal_modes[mode].sym,
-                wnScale*mol->normal_modes[mode].wavenumber);
-        normalModeAnnotation=makeAnnotation(normalModeAnnotation, CENTERED, 0.0,
-                                            1.0, 0.0, stdcol[BLACK], black, 0, 0, line);
+	annotateNormalMode(mol);
       }
       else
       {
@@ -626,8 +647,8 @@ void spectrumMouseAction(XmDrawingAreaCallbackStruct *data, int state)
           for (i=0; i<nmolecule; i++)
             restoreGeometry(molecules[i].coord, i);
         }
-        deleteAnnotation(&normalModeAnnotation);
       }
+      annotateNormalMode(mol);
     }
   } 
   else if (data->event->xbutton.button == Button2)
@@ -735,19 +756,36 @@ void selectMolecule(Widget widget, caddr_t which, XmToggleButtonCallbackStruct *
 void historyMouseAction(unsigned int button, int state, int x, int y)
 {
   struct MOLECULE *mol;
-  double xpix, ypix;
+  double xw, yw, zw;
+  int i;
 
   mol=&molecules[windows[HISTORY].set];
+  windows[HISTORY].mouseX=x;
+  windows[HISTORY].mouseY=y;
   if (button == Button1 && state == TRUE)
   {
-    pixelToWorld(HISTORY, &xpix, &ypix);
-    mol->cycle=(double)x*xpix+1.0;
-    if (mol->cycle < 1.0) mol->cycle=1.0;
-    if (mol->cycle > (double)mol->nhist) mol->cycle=(double)mol->nhist;
+    getWorldCoordinates(x, y, 0.0, &xw, &yw, &zw);
+    i=(int)floor(xw);
+    mol->cycle=xw-(double)i < (double)(i+1)-xw ? i : i+1;
+    if (xw < 1.0)
+    {
+      mol->cycle=1;
+      getScreenCoordinates(1.0, 0.0, 0.0, &xw, &yw, &zw);
+      windows[HISTORY].mouseX=(int)xw;
+    }
+    else if (xw > (double)(mol->nhist))
+    {
+      mol->cycle=mol->nhist;
+      getScreenCoordinates((double)(mol->nhist), 0.0, 0.0, &xw, &yw, &zw);
+      windows[HISTORY].mouseX=(int)xw;
+    }
   }
-  if (button == Button1 && state == FALSE)
-    mol->cycle=ceil(mol->cycle-0.5);
 
+  if (button == Button1 && state == FALSE)
+  {
+    getScreenCoordinates((double)(mol->cycle), 0.0, 0.0, &xw, &yw, &zw);
+    windows[HISTORY].mouseX=(int)xw;
+  }
   setGeometry(TRUE);
 }
 
@@ -844,14 +882,17 @@ void setGeometry(int doConnectivity)
   const GLfloat black[4] = {0.0, 0.0, 0.0, 0.0};
   double top, bottom, right, left, force;
   char line[MAXLENLINE], *word;
+  int cycle;
   register int i, j;
 
   mol=&molecules[windows[HISTORY].set];
   naSaved=mol->na;
-  j=((int)ceil(mol->cycle-0.5)-1);
-/*if (j > 0) mol->na=mol->optimization[j].natoms; */
-  i=mol->optimization[j].coords;
-  for (j=0; j<mol->na; j++, i++)
+/*cycle=((int)ceil(mol->cycle-0.5)-1);
+  if (cycle < 0) cycle=0; */
+  cycle=mol->cycle-1;
+/*if (cycle > 0) mol->na=mol->optimization[j].natoms; */
+  i=mol->optimization[cycle].coords;
+  for (j=0; j<mol->na; j++)
   {
     if (mol->atoms[j].flags & ORIGINAL)
     {
@@ -859,11 +900,24 @@ void setGeometry(int doConnectivity)
       mol->atoms[j].y=mol->history[i].y;
       mol->atoms[j].z=mol->history[i].z;
     }
+    i++;
+  }
+  if (mol->unitcell)
+  {
+    i=(int)ceil(mol->cycle-0.5);
+    for (j=0; j<mol->unitcell->nc; j++)
+    {
+      mol->unitcell[0].corners[j].x=mol->unitcell[i].corners[j].x;
+      mol->unitcell[0].corners[j].y=mol->unitcell[i].corners[j].y;
+      mol->unitcell[0].corners[j].z=mol->unitcell[i].corners[j].z;
+    }
+    expandCell(TRUE);
   }
-  if (mol->existsUnitCell) expandCell();
   word=getStringResource(topShell, "historyTitle");
-  sprintf(line, word, (int)mol->cycle, mol->optimization[(int)mol->cycle-1].energy,
-          mol->optimization[(int)mol->cycle-1].gnorm);
+/*sprintf(line, word, (int)mol->cycle, mol->optimization[(int)mol->cycle-1].energy,
+          mol->optimization[(int)mol->cycle-1].gnorm);*/
+  sprintf(line, word, mol->cycle, mol->optimization[cycle].energy,
+          mol->optimization[cycle].gnorm);
   historyAnnotation=makeAnnotation(historyAnnotation, CENTERED, 0.0, 1.0, 0.0,
                                    stdcol[BLACK], black, 0, 0, line);
 
@@ -986,6 +1040,11 @@ void deleteSelection()
     {
       for (j=0; j<molecules[i].na; j++)
         molecules[i].atoms[j].flags&=~SELECTED;
+      if (molecules[i].unitcell)
+      {
+        for (j=0; j<molecules[i].unitcell[0].nc; j++)
+          molecules[i].unitcell[0].corners[j].flags&=~SELECTED;
+      }
     }
   }
   else
@@ -993,8 +1052,26 @@ void deleteSelection()
     i=windows[VIEWER].set;
     for (j=0; j<molecules[i].na; j++)
       molecules[i].atoms[j].flags&=~SELECTED;
+    if (molecules[i].unitcell)
+    {
+      for (j=0; j<molecules[i].unitcell[0].nc; j++)
+        molecules[i].unitcell[0].corners[j].flags&=~SELECTED;
+    }
   }
   intcoord=0;
   clicked[0]=clicked[1]=clicked[2]=clicked[3]=(-1);
   showMenu=TRUE;
 }
+
+void trim(char *str)
+{
+  char *p;
+
+  p=str;
+  while (*p == ' ' || *p == '\t') p++;
+  if (p != str)
+  {
+    memmove(str, p, strlen(p));
+    str[strlen(p)]='\0';
+  }
+}
