@@ -4,12 +4,15 @@
 *                                                                              *
 *                            P R I N T F O R M . C                             *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 2000                   *
+*                 Copyright (c) Joerg-R. Hill, October 2003                    *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: printform.c,v 1.5 2000/12/10 15:13:52 jrh Exp $
+* $Id: printform.c,v 1.6 2003/11/07 11:08:33 jrh Exp $
 * $Log: printform.c,v $
+* Revision 1.6  2003/11/07 11:08:33  jrh
+* Release 2.4
+*
 * Revision 1.5  2000/12/10 15:13:52  jrh
 * Release 2.3
 *
@@ -69,6 +72,8 @@ extern char *selectFile(char *, char *, int);
 extern char *getStringResource(Widget, char *);
 extern char *saveVectorFile(int, char *, int);
 extern char *saveTiff(int, char *, int);
+extern char *savePng(int, char *, int);
+extern char *savePostScriptBitmap(int, char *);
 extern void GetMessageBoxButton(Widget, XtPointer, caddr_t);
 extern void setAnimation(int);
 extern Boolean normalMode(XtPointer);
@@ -77,6 +82,7 @@ extern int messgb(Widget, int, char *, struct PushButtonRow *, int);
 extern void CreatePushButtonRow(Widget, struct PushButtonRow *, int);
 extern void setMenuItem(int, int);
 extern Widget initShell(Widget, char *, Widget *, Widget *);
+extern void GetTransperancy(Widget, caddr_t, XmToggleButtonCallbackStruct *);
 
 extern struct WINDOW windows[];
 extern struct MOLECULE *molecules;
@@ -84,9 +90,9 @@ extern Widget topShell, fileBox;
 extern double paperWidth, paperHeight;
 extern int format, landscape, animate, primitive, iwavef;
 
-static Widget filename, dialog, pWidth, pHeight, comp, paperformat;
+static Widget filename, dialog, pWidth, pHeight, comp, transparent, paperformat;
 static int window, format_save, landscape_save, compression=COMPRESSION_NONE;
-static int paper=A4;
+static int transparentBackground=FALSE, paper=A4;
 static char name[MAXLENLINE];
 static struct PushButtonRow radiobox2_buttons[] = {
   { "landscape", GetOrientation, (XtPointer)TRUE, NULL },
@@ -104,6 +110,7 @@ void printDialog(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
   };
   static struct PushButtonRow radiobox1_buttons[] = {
     { "tiff", GetFormat, (XtPointer)TIFFFILE, NULL },
+    { "png", GetFormat, (XtPointer)PNGFILE, NULL },
     { "hpgl", GetFormat, (XtPointer)HPGL, NULL },
     { "postscript", GetFormat, (XtPointer)POSTSCRIPT, NULL },
     { "raytracer", GetFormat, (XtPointer)RAYTRACER, NULL }
@@ -226,6 +233,18 @@ void printDialog(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
     XtVaSetValues(comp, XmNsensitive, False, NULL);
   XtManageChild(comp);
 
+  transparent=XtVaCreateManagedWidget("transparent", xmToggleButtonWidgetClass, form2,
+                                      XmNleftAttachment, XmATTACH_FORM,
+                                      XmNtopAttachment, XmATTACH_WIDGET,
+						  XmNtopWidget, comp,
+                                      XmNrightAttachment, XmATTACH_FORM,
+                                      NULL);
+  if (transparentBackground) XtVaSetValues(transparent, XmNset, True, NULL);
+  XtAddCallback(transparent, XmNvalueChangedCallback, (XtCallbackProc)GetTransperancy,
+                &transparentBackground);
+  if (format != PNGFILE) 
+    XtVaSetValues(transparent, XmNsensitive, False, NULL);
+
   sep3=XtVaCreateManagedWidget("sep3", xmSeparatorWidgetClass, form,
                                XmNorientation, XmHORIZONTAL,
                                XmNtraversalOn, False,
@@ -245,6 +264,8 @@ void printDialog(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
   {
     case TIFFFILE:   strcpy(name, "vm_image.tiff");
                      break;
+    case PNGFILE:    strcpy(name, "vm_image.png");
+                     break;
     case HPGL:       strcpy(name, "vm_plot.hpgl");
                      break;
     case POSTSCRIPT: strcpy(name, "vm_plot.eps");
@@ -287,7 +308,7 @@ void printDialog(Widget widget, caddr_t which, XmAnyCallbackStruct *data)
                                  XmNbottomAttachment, XmATTACH_FORM,
                                  NULL);
   }
-  if (format == TIFFFILE || format == RAYTRACER)
+  if (format == TIFFFILE || format == PNGFILE || format == RAYTRACER)
     XtVaSetValues(paperformat, XmNsensitive, False, NULL);
   XtManageChild(paperformat);
 
@@ -364,24 +385,35 @@ void GetFormat(Widget button, caddr_t which, XmToggleButtonCallbackStruct *data)
                      XtVaSetValues(radiobox2_buttons[1].widget, XmNsensitive, False, NULL);
                      XtVaSetValues(comp, XmNsensitive, True, NULL);
                      XtVaSetValues(paperformat, XmNsensitive, False, NULL);
+                     XtVaSetValues(transparent, XmNsensitive, False, NULL);
+                     break;
+    case PNGFILE:    strcpy(name, "vm_image.png");
+                     XtVaSetValues(radiobox2_buttons[0].widget, XmNsensitive, False, NULL);
+                     XtVaSetValues(radiobox2_buttons[1].widget, XmNsensitive, False, NULL);
+                     XtVaSetValues(comp, XmNsensitive, True, NULL);
+                     XtVaSetValues(paperformat, XmNsensitive, False, NULL);
+                     XtVaSetValues(transparent, XmNsensitive, True, NULL);
                      break;
     case HPGL:       strcpy(name, "vm_plot.hpgl");
                      XtVaSetValues(radiobox2_buttons[0].widget, XmNsensitive, True, NULL);
                      XtVaSetValues(radiobox2_buttons[1].widget, XmNsensitive, True, NULL);
                      XtVaSetValues(comp, XmNsensitive, False, NULL);
                      XtVaSetValues(paperformat, XmNsensitive, True, NULL);
+                     XtVaSetValues(transparent, XmNsensitive, False, NULL);
                      break;
     case POSTSCRIPT: strcpy(name, "vm_plot.eps");
                      XtVaSetValues(radiobox2_buttons[0].widget, XmNsensitive, True, NULL);
                      XtVaSetValues(radiobox2_buttons[1].widget, XmNsensitive, True, NULL);
                      XtVaSetValues(comp, XmNsensitive, False, NULL);
                      XtVaSetValues(paperformat, XmNsensitive, True, NULL);
+                     XtVaSetValues(transparent, XmNsensitive, False, NULL);
                      break;
     case RAYTRACER:  strcpy(name, "vm_image.ray");
                      XtVaSetValues(radiobox2_buttons[0].widget, XmNsensitive, False, NULL);
                      XtVaSetValues(radiobox2_buttons[1].widget, XmNsensitive, False, NULL);
                      XtVaSetValues(comp, XmNsensitive, False, NULL);
                      XtVaSetValues(paperformat, XmNsensitive, False, NULL);
+                     XtVaSetValues(transparent, XmNsensitive, False, NULL);
                      break;
   }
   XtVaSetValues(filename, XmNvalue, name, NULL);
@@ -399,6 +431,14 @@ void GetOrientation(Widget button, caddr_t which, XmToggleButtonCallbackStruct *
   }
 }
 
+void GetTransperancy(Widget button, caddr_t which, XmToggleButtonCallbackStruct *data)
+{
+  if (data->set)
+    *(int *)which=TRUE;
+  else
+    *(int *)which=FALSE;
+}
+
 void GetPapersize(Widget button, caddr_t which, XmToggleButtonCallbackStruct *data)
 {
   paper=(int)which;
@@ -512,8 +552,18 @@ char *saveFile(struct MOLECULE *mol, int format, char *file, char *f)
                      else
                        word=saveTiff(window, file, compression);
                      break;
-    case HPGL:
-    case POSTSCRIPT: word=saveVectorFile(window, file, format);
+    case PNGFILE:    if (((animate == ANIMATE && mol->mode != (-1))
+                        || animate == TRAJECTORY) && window == VIEWER)
+			     word=saveMultipleFiles(mol, file, f, animate, format);
+                     else
+                       word=savePng(window, file, transparentBackground);
+                     break;
+    case HPGL:       word=saveVectorFile(window, file, format);
+                     break;
+    case POSTSCRIPT: if (window == VIEWER && primitive == GLU_FILL && (windows[VIEWER].mode != WIREMODEL || iwavef != ALL_OFF))
+                       word=savePostScriptBitmap(window, file);
+		     else
+                        word=saveVectorFile(window, file, format);
                      break;
     case RAYTRACER:  if (((animate == ANIMATE && mol->mode != (-1))
                         || animate == TRAJECTORY) && window == VIEWER)
@@ -589,9 +639,11 @@ char *saveMultipleFiles(struct MOLECULE *mol, char *file, char *f, int animate,
      switch (format)
      {
        case TIFFFILE:  word=saveTiff(window, f, compression);
- 	                 break;
+                       break;
+       case PNGFILE:   word=savePng(window, f, compression);
+                       break;
        case RAYTRACER: word=saveVectorFile(window, f, format);
-	                 break;
+                       break;
      }
      j=0;
   }
