@@ -4,12 +4,15 @@
 *                                                                              *
 *                            R E A D G A U S S . C                             *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 2000                   *
+*                 Copyright (c) Joerg-R. Hill, October 2003                    *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: readgauss.c,v 1.5 2000/12/10 15:15:24 jrh Exp $
+* $Id: readgauss.c,v 1.6 2003/11/07 11:13:35 jrh Exp $
 * $Log: readgauss.c,v $
+* Revision 1.6  2003/11/07 11:13:35  jrh
+* Release 2.4
+*
 * Revision 1.5  2000/12/10 15:15:24  jrh
 * Release 2.3
 *
@@ -39,7 +42,7 @@
 #define MAXLENLINE 134
 #define TABMAR "---"
 #define EHF "rhf) ="
-#define EMP2 "eump2="
+#define EMP2 "eump2"
 #define EMP4 "sdtq)="
 #define EDFT ") ="
 #define EUHF "uhf) ="
@@ -53,9 +56,11 @@
 #define OSYM "orbital symmetries"
 #define AMOS "alpha mos:"
 #define BMOS "beta  mos:"
+#define MOCOEFFICIENTS "molecular orbital coefficients"
 #define AXES "***** axes restored to original set *****"
-#define SYMMETRIES 1
-#define ALPHAMOS   2
+#define SYMMETRIES   1
+#define ALPHAMOS     2
+#define COEFFICIENTS 3
 
 int  main(int, char **);
 void readBasis(FILE *, int);
@@ -76,15 +81,15 @@ extern void eof(char *, char *, int);
 int main(int argc, char **argv)
 {
   FILE *file;
-/*double etotal;*/
+  double etotal, *xcoord, *ycoord, *zcoord;
   char emark[2][7] = {"0@  ~0", "0@  ~0"}, line[MAXLENLINE], *word;
   char *symbol, orientation[24];
   int freq=FALSE, gfprn=FALSE, opt=FALSE;
   int nemark=0, nout, na, ncent, nuc;
   int mna=50, cycle=0;
-/*int etex; */
+  int etex;
   register double x, y, z;
-  register int i; /* mak;*/
+  register int i, mak;
 
   if ((file=fopen(argv[1], "r")) == NULL) eof("noFile", argv[1], 1);
 
@@ -151,7 +156,7 @@ int main(int argc, char **argv)
 
     if (strstr(line, "lyp") || strstr(line, "pl") || strstr(line, "andh") ||
         strstr(line, "hfb") || strstr(line, "xalpha") || strstr(line, "hfs") ||
-        strstr(line, "p86") || strstr(line, "vwn"))
+        strstr(line, "p86") || strstr(line, "vwn") || strstr(line, "pw91"))
     {
       strcpy(emark[0], EDFT);
       nemark=1;
@@ -206,6 +211,9 @@ int main(int argc, char **argv)
   na=0;
   ncent=0;
   if (!opt) printf("$coord 1.0\n");
+  xcoord=(double *)getmem(mna, sizeof(double));
+  ycoord=(double *)getmem(mna, sizeof(double));
+  zcoord=(double *)getmem(mna, sizeof(double));
   symbol=(char *)getmem(mna, 3*sizeof(char));
   while (fgets(line, MAXLENLINE, file) != NULL)
   {
@@ -226,10 +234,16 @@ int main(int argc, char **argv)
       symbol[3*na]=pse[nuc][0];
       symbol[3*na+1]=pse[nuc][1];
       symbol[3*na+2]='\0';
+	xcoord[na]=x;
+	ycoord[na]=y;
+	zcoord[na]=z;
       if (!opt) printf("%22.14f%22.14f%22.14f  %s\n", x, y, z, &symbol[3*na]);
       if (++na >= mna)
       {
         mna+=50;
+        xcoord=(double *)expmem((void *)xcoord, mna, sizeof(double));
+        ycoord=(double *)expmem((void *)ycoord, mna, sizeof(double));
+        zcoord=(double *)expmem((void *)zcoord, mna, sizeof(double));
         symbol=(char *)expmem((void *)symbol, mna, 3*sizeof(char));
       }
     }
@@ -254,13 +268,22 @@ int main(int argc, char **argv)
 
 /* Load total energy */
 
-/*do
+  if (!opt)
   {
-    if (fgets(line, MAXLENLINE, file) == NULL) eof("noEnergy", argv[1], 0);
-    makeLower(line);
-  } while ((mak=msub(line, nemark, &emark[0][0])) == 0);
-  etotal=reade(line, emark[mak], &etex);
-  if (!etex) exit(-1); */
+    do
+    {
+      if (fgets(line, MAXLENLINE, file) == NULL) eof("noEnergy", argv[1], 0);
+      makeLower(line);
+    } while ((mak=msub(line, nemark, &emark[0][0])) == 0);
+    etotal=reade(line, emark[mak], &etex);
+    printf("$grad\n");
+    printf("  cycle =   1    SCF energy = %17.10f   |dE/xyz| = 0.0\n", etotal);
+    for (i=0; i<na; i++)
+      printf("%22.14f%22.14f%22.14f  %s\n", xcoord[i], ycoord[i], zcoord[i], &symbol[3*i]);
+    for (i=0; i<na; i++)
+      printf("%22.14e%22.14e%22.14e\n", 0.0, 0.0, 0.0);
+/*  if (!etex) exit(-1);*/
+  }
 
 /* Load orbital energies and symmetries */
 
@@ -277,6 +300,10 @@ int main(int argc, char **argv)
 
   printf("$end\n");
   fclose(file);
+  fremem((void **)&xcoord);
+  fremem((void **)&ycoord);
+  fremem((void **)&zcoord);
+  fremem((void **)&symbol);
   return(0);
 }
 
@@ -438,12 +465,12 @@ void readMOs(int cycle, FILE *file)
       if (strstr(line, "the electronic state") || strstr(line, " alpha deviation from unit magnitude")) break;
       if (strstr(line, "occupied") || strstr(line, "virtual"))
       {
-            if (strstr(line, "virtual")) occ=FALSE;
-          strtok(line, " \t");
+        if (strstr(line, "virtual")) occ=FALSE;
+        strtok(line, " \t");
         begin=NULL;
       }
       else
-          begin=line;
+        begin=line;
       while ((word=strtok(begin, " \t")) != NULL)
       {
         begin=NULL;
@@ -456,7 +483,7 @@ void readMOs(int cycle, FILE *file)
           {
             if (!strncmp(&symorb[4*i], &closed[MAXLENLINE*j], strlen(&symorb[4*i])))
             {
-                sprintf(number, ",%d", i+1);
+              sprintf(number, ",%d", i+1);
               strcat(&closed[MAXLENLINE*j], number);
               found=TRUE;
               break;
@@ -471,7 +498,7 @@ void readMOs(int cycle, FILE *file)
         i++;
         if (i >= mno)
         {
-              mno+=20;
+          mno+=20;
           symorb=(char *)expmem((void *)symorb, mno, 4*sizeof(char));
         }
       }
@@ -505,6 +532,12 @@ void readMOs(int cycle, FILE *file)
     makeLower(line);
     word=strstr(line, AMOS);
     if (word) cycle--;
+    if (!word)
+    {
+      word=strstr(line, MOCOEFFICIENTS);
+      if (word)
+	info=COEFFICIENTS;
+    }
   } while (!word || cycle > 0);
 
   printf("$scfmo gaussian\n");
@@ -518,6 +551,8 @@ void readMOs(int cycle, FILE *file)
     while (strtok(NULL, " \t") != NULL) ie++;
     ie++;
     fgets(line, MAXLENLINE, file);
+    if (info == COEFFICIENTS)
+      fgets(line, MAXLENLINE, file);
     strtok(line, " \t");
     strtok(NULL, " \t");
     for (i=ia; i<ia+ie; i++)
@@ -539,15 +574,17 @@ void readMOs(int cycle, FILE *file)
     for (j=0; j<norbital; j++)
     {
       fgets(line, MAXLENLINE, file);
-      strtok(line, " \t");
+      word=strtok(line, " \t");
+      while (!strchr(word, '.')) word=strtok(NULL, " \t");
       for (i=0; i<ie; i++)
       {
-        cmo[k]=atof(strtok(NULL, " \t"));
+        cmo[k]=atof(word);
         if (++k >= mnc)
         {
           mnc+=200;
           cmo=(double *)expmem((void *)cmo, mnc, sizeof(double));
-          }
+        }
+        word=strtok(NULL, " \t");
       }
     }
     for (i=ia; i<ia+ie; i++)
@@ -558,7 +595,7 @@ void readMOs(int cycle, FILE *file)
       for (j=0; j<norbital; j++)
       {
         printf(" %19.14e", cmo[i-ia+j*offset]);
-          if ((j+1) % 4 == 0) printf("\n");
+        if ((j+1) % 4 == 0) printf("\n");
       }
       if (j % 4 != 0) printf("\n");
     }
@@ -566,7 +603,7 @@ void readMOs(int cycle, FILE *file)
     makeLower(line);
     ia+=ie;
   }
-  while (!strstr(line, BMOS));
+  while (!strstr(line, BMOS) && !strstr(line, "condensed to atoms"));
 
   printf("$closed shells\n");
   for (i=0; i<nocc; i++)
