@@ -4,12 +4,15 @@
 *                                                                              *
 *                              V I E W M O L . C                               *
 *                                                                              *
-*                 Copyright (c) Joerg-R. Hill, December 2000                   *
+*                 Copyright (c) Joerg-R. Hill, October 2003                    *
 *                                                                              *
 ********************************************************************************
 *
-* $Id: viewmol.c,v 1.5 2000/12/10 15:17:58 jrh Exp $
+* $Id: viewmol.c,v 1.6 2003/11/07 11:17:09 jrh Exp $
 * $Log: viewmol.c,v $
+* Revision 1.6  2003/11/07 11:17:09  jrh
+* Release 2.4
+*
 * Revision 1.5  2000/12/10 15:17:58  jrh
 * Release 2.3
 *
@@ -26,6 +29,7 @@
 * Initial revision
 *
 */
+#include<locale.h>
 #include<signal.h>
 #include<stdio.h>
 #include<X11/Xlib.h>
@@ -58,8 +62,8 @@ void installColormap(void);
 
 extern void drawMolecule(Widget, caddr_t, XmDrawingAreaCallbackStruct *);
 extern void processInput(Widget, caddr_t, XmDrawingAreaCallbackStruct *);
-extern void printString(char *, double, double, double, double, GLuint);
-extern void feedbackString(char *, double, double, double, double, GLuint);
+extern void printString(char *, double, double, double, double, GLuint, char);
+extern void feedbackString(char *, double, double, double, double, GLuint, char);
 extern void feedbackLineStipple(GLint, GLushort);
 extern void feedbackLineWidth(GLfloat);
 extern void feedbackDisable(GLenum);
@@ -69,6 +73,7 @@ extern void raytracerColor4fv(const GLfloat *);
 extern void raytracerClearColor(GLclampf, GLclampf, GLclampf, GLclampf);
 extern void raytracerSphere(GLUquadricObj *, GLdouble, GLint, GLint);
 extern void raytracerCylinder(GLUquadricObj *qobj, GLdouble, GLdouble, GLdouble, GLint, GLint);
+extern void raytracerCone(GLUquadricObj *qobj, GLdouble, GLdouble, GLdouble, GLint, GLint);
 extern void GetMessageBoxButton(Widget, XtPointer, caddr_t);
 extern char *getStringResource(Widget, char *);
 extern void pixelToWorld(int, double *, double *);
@@ -89,6 +94,9 @@ extern Colormap getRGBcolormap(Display *, int, XVisualInfo *);
 extern void incrementQuaternion(double, double, double, float *);
 extern void initPython(int, char **);
 extern void loadAtomColors(XVisualInfo *, Colormap);
+extern int findShadowMapSupport(void);
+
+extern void testBegin(GLenum), testEnd(void), testVertex(double, double, double), testNorm(double, double, double);
 
 void (*drawBegin)(GLenum), (*drawEnd)(void), (*drawVertex3d)(double, double, double);
 void (*drawVertex2d)(double, double), (*drawNormal3d)(double, double, double);
@@ -96,7 +104,8 @@ void (*drawColor4fv)(const GLfloat *);
 void (*drawClearColor)(GLclampf, GLclampf, GLclampf, GLclampf);
 void (*drawSphere)(GLUquadricObj *, GLdouble, GLint, GLint);
 void (*drawCylinder)(GLUquadricObj *qobj, GLdouble, GLdouble, GLdouble, GLint, GLint);
-void (*drawString)(char *, double, double, double, double, GLuint);
+void (*drawCone)(GLUquadricObj *qobj, GLdouble, GLdouble, GLdouble, GLint, GLint);
+void (*drawString)(char *, double, double, double, double, GLuint, char);
 void (*drawLineStipple)(GLint, GLushort);
 void (*drawLineWidth)(GLfloat);
 void (*drawDisable)(GLenum);
@@ -115,7 +124,7 @@ PyObject *viewmolModule, *moleculeModule;
 double *grid=NULL;
 double bndfac=1.0, amplitude, forceScale, wnScale=1.0, hbondThreshold=2.0, scaleRadius=1.0;
 double tmat[4][4];
-double paperWidth=0.0, paperHeight=0.0, sphereres=10., lineWidth=0.0;
+double paperWidth=0.0, paperHeight=0.0, sphereres=10., lineWidth=0.0, spectrumLineWidth=1.0;
 double temp, pressure=1.0, weight=0.0;
 double e1, e2, denres=(1.);
 double level=0.05, gridres=(-1.), torsionDefault=180.0, drawingSpeed=0.0;
@@ -127,13 +136,13 @@ int ne, nopt=0, noutput=0, nUndo=0;
 int nmolecule=0, imol, ireaction=0;
 int rotateXY=FALSE, rotateZ=FALSE;
 int primitive, animate=ANIMATE, lights=3, moveItem=MOLECULES, projectionMode=ORTHO;
-int lines=TRUE, showForces=FALSE, setins=FALSE;
+int lines=TRUE, showForces=FALSE, setins=FALSE, xIsTop=TRUE, showGrid=TRUE;
 int showUnitCell=TRUE, bondOrderDefault=1;
 int showInertia=FALSE, selectAtom=FALSE, simplify=TRUE;
 int rgbMode, picking=FALSE, swapBuffers=TRUE, shadows=FALSE, ground=TRUE, drawIcon=FALSE;
 int drawingDevice, bondType=CONJUGATION, showHBond=TRUE, localGeometry=TRUE;
 int label, outputType=0, automaticRecalculate=FALSE;
-int debug, annotate=FALSE, saveAll=TRUE;
+int showWarning=TRUE, debug, annotate=FALSE, saveAll=TRUE;
 int nAnnotations=0, normalModeAnnotation=(-1), selectAtomAnnotation=(-1);
 int historyAnnotation=(-1), setinsAnnotation=(-1);
 int unit, bfatom, element=0, modifyMolecule=NOTHING;
@@ -142,11 +151,12 @@ int spaceballButtonPress, spaceballMotionNotify;
 int showMenu=TRUE, saveLanguage=FALSE;
 int iwavef=ALL_OFF, interp=IP_LINEAR;
 int format=TIFFFILE, landscape=TRUE;
+int shadowMap=FALSE;
 char *formBondLength, *formBondAngle, *formTorsionAngle;
-char *text, *textPointer;
+char *text, *textPointer, *moduleName;
 char language[20];
 char webbrowser[MAXLENLINE]={""}, moloch[MAXLENLINE]={""};
-char rayshade[MAXLENLINE]={""}, displayRLE[MAXLENLINE]={""};
+char raytracer[MAXLENLINE]={""}, displayImage[MAXLENLINE]={""};
 pid_t manual_pid=0, raypid=0;
 
 int main(int argc, char** argv)
@@ -174,8 +184,9 @@ int main(int argc, char** argv)
 /*Atom dropType;
   int dropBufferSize, dropX, dropY;
   unsigned char *dropBuffer; */
-  char *line;
+  char *line, *locale;
 
+  locale=setlocale(LC_ALL, "");
   signal(SIGBUS,  signalHandler);
   signal(SIGFPE,  signalHandler);
   signal(SIGSEGV, signalHandler);
@@ -193,14 +204,24 @@ int main(int argc, char** argv)
   if ((vi=glXChooseVisual(display, XScreenNumberOfScreen(XtScreen(topShell)),
        attrib)) == NULL || vi->class == DirectColor)
   {
-    attrib[9]=None;
-    rgbMode=FALSE;
+    /* Could not find a double buffered RGB visual with stencil buffer,
+       try to find one without stencil buffer */
+    attrib[10]=None;
     if ((vi=glXChooseVisual(display, XScreenNumberOfScreen(XtScreen(topShell)),
          attrib)) == NULL)
     {
-      line=getStringResource(topShell, "noVisual");
-      messgb(topShell, 3, line, buttons, 1);
-      exit(-1);
+      /* Could not find a double buffered RGB visual, try to find a
+         color mapped one */
+      attrib[9]=None;
+      rgbMode=FALSE;
+      if ((vi=glXChooseVisual(display, XScreenNumberOfScreen(XtScreen(topShell)),
+           attrib)) == NULL)
+      {
+	/* Still no success, give up */
+        line=getStringResource(topShell, "noVisual");
+        messgb(topShell, 3, line, buttons, 1);
+        exit(-1);
+      }
     }
   }
   colormap=getRGBcolormap(display, vi->screen, vi);
@@ -213,9 +234,15 @@ int main(int argc, char** argv)
     XtVaSetValues(topShell, XtNvisual, vi->visual, XtNdepth, vi->depth, NULL);*/
   XtRealizeWidget(topShell);
 
+  windows[VIEWER].left=-1.0;
+  windows[VIEWER].right=1.0;
+  windows[VIEWER].bottom=-1.0;
+  windows[VIEWER].top=1.0;
+  windows[VIEWER].near=-1.0;
+  windows[VIEWER].far=1.0;
   windows[VIEWER].widget=XtVaCreateManagedWidget("viewer", xmDrawingAreaWidgetClass, topShell,
                                                  XtNvisual, vi->visual,
-								 XmNtranslations, XtParseTranslationTable(translations),
+						 XmNtranslations, XtParseTranslationTable(translations),
                                                  XmNmarginHeight, 0,
                                                  XmNmarginWidth, 0,
                                                  NULL);
@@ -228,7 +255,7 @@ int main(int argc, char** argv)
   findInputDevices(windows[VIEWER].widget);
 /*XtAddEventHandler(topShell, PropertyChangeMask, False,
                     (XtEventHandler)setIcon, (XtPointer)NULL); */
-
+  shadowMap=findShadowMapSupport();
   if (getrc())
   {
     setDrawingDevice(SCREEN);
@@ -269,6 +296,7 @@ void reshape(Widget widget, caddr_t window, XmDrawingAreaCallbackStruct *data)
   XtVaGetValues(widget, XmNwidth, &width, XmNheight, &height, NULL);
   glXMakeCurrent(XtDisplay(windows[w].widget), XtWindow(windows[w].widget), windows[w].context);
   setShape((int)window, width, height);
+  redraw(w);
 }
 
 void setShape(int window, Dimension width, Dimension height)
@@ -276,7 +304,7 @@ void setShape(int window, Dimension width, Dimension height)
   double rel;
 
   XtVaSetValues(windows[window].widget, XmNwidth, width, XmNheight, height,
-		    NULL);
+                NULL);
   glViewport(0, 0, width, height);
   if (window == VIEWER)
   {
@@ -337,6 +365,7 @@ void getRotation(int x, int y, int z, int mode)
 void getTranslation(int x, int y, int mode)
 {
   static int xold, yold;
+  Dimension width, height;
   double xpix, ypix;
 
   if (rotateXY == 1)
@@ -344,15 +373,16 @@ void getTranslation(int x, int y, int mode)
     pixelToWorld(VIEWER, &xpix, &ypix);
     if (moveItem & 0x80000000)
     {
+      XtVaGetValues(windows[VIEWER].widget, XmNwidth, &width, XmNheight, &height, NULL);
       if (mode == 1)
       {
-        annotation[moveItem & 0x7fffffff].x+=(double)x*xpix;
-        annotation[moveItem & 0x7fffffff].y+=(double)y*ypix;
+        annotation[moveItem & 0x7fffffff].x+=2.0*(double)x/(double)width;
+        annotation[moveItem & 0x7fffffff].y+=2.0*(double)y/(double)height;
       }
       else
       {
-        annotation[moveItem & 0x7fffffff].x+=(double)(x-xold)*xpix;
-        annotation[moveItem & 0x7fffffff].y+=(double)(yold-y)*ypix;
+        annotation[moveItem & 0x7fffffff].x+=2.0*(double)(x-xold)/(double)width;
+        annotation[moveItem & 0x7fffffff].y+=2.0*(double)(yold-y)/(double)height;
       }
     }
     else
@@ -422,6 +452,7 @@ void setDrawingDevice(int device)
                      drawClearColor=glClearColor;
                      drawSphere=gluSphere;
                      drawCylinder=gluCylinder;
+			   drawCone=gluCylinder;
                      drawString=printString;
                      drawLineStipple=glLineStipple;
                      drawLineWidth=glLineWidth;
@@ -440,6 +471,7 @@ void setDrawingDevice(int device)
                      drawDisable=glDisable;
                      drawSphere=gluSphere;
                      drawCylinder=gluCylinder;
+			   drawCone=gluCylinder;
                      drawString=feedbackString;
                      drawLineStipple=feedbackLineStipple;
                      drawLineWidth=feedbackLineWidth;
@@ -454,6 +486,7 @@ void setDrawingDevice(int device)
                      drawClearColor=raytracerClearColor;
                      drawSphere=raytracerSphere;
                      drawCylinder=raytracerCylinder;
+                     drawCone=raytracerCone;
                      drawString=printString;
                      drawLineStipple=glLineStipple;
                      drawLineWidth=glLineWidth;
@@ -486,6 +519,7 @@ void ende(Widget widget, caddr_t client_data, XmAnyCallbackStruct *call_data)
   }
   if (options != NULL) fremem((void **)&options);
   fremem((void **)&molecules);
+  nmolecule=0;
   if (manual_pid && kill(manual_pid, 0) != -1) kill(manual_pid, SIGTERM);
   if (raypid && kill(raypid, 0) != -1) kill(raypid, SIGTERM);
   Py_Exit(0);
@@ -508,7 +542,7 @@ void setIcon(Widget widget, XtPointer dummy, XEvent *e)
   int count, restore=FALSE;
   unsigned int w, h;
 
-#ifdef LINUX
+#if defined LINUX || defined DARWIN || defined FREEBSD
   wmStateAtom=XInternAtom(XtDisplay(topShell), "XmSWM_STATE", False);
 #else
   wmStateAtom=XSGIFastInternAtom(XtDisplay(topShell), "XmSWM_STATE",
